#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from casuarius import Solver, medium

from ..guard import guard


class ConstraintsLayout(object):
    """ A class which uses a casuarius solver to manage a system 
    of constraints.

    """ 
    #: The internal Casuarius solver instance to use for this layout.
    _solver = None

    #: The internal flag indicating if the solver is initialized
    _initialized = False

    @property
    def initialized(self):
        """ A read-only property which returns whether or not this solver
        instance has been initialized.

        """
        return self._initialized

    def initialize(self, constraints):
        """ Initializes the solver by creating all of the necessary 
        constraints for this components layout children and adding them
        to the solver.

        Parameters
        ----------
        constraints : Iterable
            An iterable that yields the constraints to add to the
            solvers.

        """
        self._initialized = False
        self._solver = solver = Solver(autosolve=False)
        for cn in constraints:
            solver.add_constraint(cn)
        solver.autosolve = True
        self._initialized = True

    def update_constraints(self, old_cns, new_cns):
        """ Re-run the initialization routine to build a new solver with
        updated constraints. This should typically only be called when 
        the user constraints are updated.

        Parameters
        ----------
        old_cns : list
            The list of constraints to remove from the solver.
        
        new_cns : list
            The list of constraints to add to the solver.

        """
        if not self._initialized:
            raise RuntimeError('Update constraints on uninitialized solver')

        solver = self._solver
        solver.autosolve = False
        for cn in old_cns:
            solver.remove_constraint(cn)
        for cn in new_cns:
            solver.add_constraint(cn)
        solver.autosolve = True

    def layout(self, cb, width, height, size, strength=medium, weight=1.0):
        """ Perform an iteration of the solver for the new width and 
        height of the component.

        Parameters
        ----------
        cb : callable
            A callback which will be called when new values from the
            solver are available. This will be called from within a 
            solver context while the solved values are valid. Thus 
            the new values should be consumed before the callback
            returns.

        width : Constraint Variable
            The constraint variable representing the width of the
            main layout container.
        
        height : Constraint Variable
            The constraint variable representing the height of the
            main layout container.

        size : (int, int)
            The (width, height) size tuple which is the current size
            of the main layout container.

        strength : casuarius strength, optional
            The strength with which to perform the layout using the
            current size of the container. i.e. the strength of the
            resize. The default is casuarius.medium.

        weight : float, optional
            The weight to apply to the strength. The default is 1.0

        """
        if not self._initialized:
            raise RuntimeError('Layout with uninitialized solver')

        if not guard.guarded(self, 'layout'):
            with guard(self, 'layout'):
                w, h = size
                values = [(width, w), (height, h)]
                with self._solver.suggest_values(values, strength, weight):
                    cb()

    def get_min_size(self, width, height, strength=medium, weight=0.1):
        """ Run an iteration of the solver with the suggested size of the
        component set to (0, 0). This will cause the solver to effectively
        compute the minimum size that the window can be to solve the
        system. 

        Parameters
        ----------
        width : Constraint Variable
            The constraint variable representing the width of the
            main layout container.
        
        height : Constraint Variable
            The constraint variable representing the height of the
            main layout container.

        strength : casuarius strength, optional
            The strength with which to perform the layout using the
            current size of the container. i.e. the strength of the
            resize. The default is casuarius.medium.
        
        weight : float, optional
            The weight to apply to the strength. The default is 0.1
            so that constraints of medium strength but default weight
            have a higher precedence than the minimum size.
        
        Returns
        -------
        result : (float, float)
            The floating point (min_width, min_height) size of the 
            container which would best satisfy the set of constraints.

        """
        if not self._initialized:
            raise RuntimeError('Get min size on uninitialized solver')

        values = [(width, 0.0), (height, 0.0)]
        with self._solver.suggest_values(values, strength, weight):
            min_width = width.value
            min_height = height.value
        return (min_width, min_height)

    def get_max_size(self, width, height, strength=medium, weight=0.1):
        """ Run an iteration of the solver with the suggested size of 
        the component set to a very large value. This will cause the 
        solver to effectively compute the maximum size that the window
        can be to solve the system. The return value is a tuple numbers.
        If one of the numbers is -1, it indicates there is no maximum in
        that direction.

        Parameters
        ----------
        width : Constraint Variable
            The constraint variable representing the width of the
            main layout container.
        
        height : Constraint Variable
            The constraint variable representing the height of the
            main layout container.

        strength : casuarius strength, optional
            The strength with which to perform the layout using the
            current size of the container. i.e. the strength of the
            resize. The default is casuarius.medium.
        
        weight : float, optional
            The weight to apply to the strength. The default is 0.1
            so that constraints of medium strength but default weight
            have a higher precedence than the minimum size.
        
        Returns
        -------
        result : (float or -1, float or -1)
            The floating point (max_width, max_height) size of the 
            container which would best satisfy the set of constraints.

        """
        if not self._initialized:
            raise RuntimeError('Get max size on uninitialized solver')

        max_val = 2**24 - 1 # Arbitrary, but the max allowed by Qt.
        values = [(width, max_val), (height, max_val)]
        with self._solver.suggest_values(values, strength, weight): 
            max_width = width.value
            max_height = height.value
        width_diff = abs(max_val - int(round(max_width)))
        height_diff = abs(max_val - int(round(max_height)))
        if width_diff <= 1:
            max_width = -1
        if height_diff <= 1:
            max_height = -1
        return (max_width, max_height)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .qt_test_assistant import QtTestAssistant
from .. import calendar

from enaml.backends.qt.qt_date_edit import qdate_to_python


class TestQtCalendar(QtTestAssistant, calendar.TestCalendar):
    """ QtCalendar tests. 

    """
    def get_date(self, widget):
        """  Get the toolkits widget's active date.

        """
        date = widget.selectedDate()
        return qdate_to_python(date)

    def get_min_date(self, widget):
        """  Get the toolkits widget's maximum date attribute.

        """
        date = widget.minimumDate()
        return qdate_to_python(date)

    def get_max_date(self, widget):
        """ Get the toolkits widget's minimum date attribute.

        """
        date = widget.maximumDate()
        return qdate_to_python(date)

    def activate_date(self, widget, date):
        """ Fire an event to indicate that a date was activated.

        """
        widget.activated.emit(date)

    def select_date(self, widget, date):
        """ Fire an event to indicate that a date was selected.

        """
        # The setDate method in Qt will also signal the dateSelected 
        # signal.
        widget.setSelectedDate(date)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from datetime import date

from traits.api import TraitError

from .enaml_test_case import EnamlTestCase, required_method


class TestCalendar(EnamlTestCase):
    """ Logic for testing calendars.

    Toolkit testcases need to provide the following functions

    Abstract Methods
    ----------------
    get_date(self, widget)
        Get a calendar's active date.

    get_min_date(self, widget)
        Get a calendar's minimum date attribute.

    get_max_date(self, widget)
        Get a calendar's maximum date attribute.

    activate_date(self, widget, date)
        Fire an event to indicate that a date was activated.

    select_date(self, widget, date)
        Fire an event to indicate that a date was selected.

    """

    def setUp(self):
        """ Set up before the calendar tests

        """

        source = """
enamldef MainView(MainWindow):
    attr events
    Calendar:
        name = 'cal'
        selected :: events.append(('selected', event.new))
        activated :: events.append(('activated', event.new))
"""

        self.events = []
        self.view = self.parse_and_create(source, events=self.events)
        self.component = self.component_by_name(self.view, 'cal')
        self.widget = self.component.toolkit_widget

    def test_initial_value(self):
        """ Test the initial attributes of the calendar.

        """
        component = self.component

        self.assertEnamlInSync(component, 'date', date.today())
        self.assertEnamlInSync(component, 'min_date', date(1752, 9, 14))
        self.assertEnamlInSync(component, 'max_date', date(7999, 12, 31))
        self.assertEqual(self.events, [])

    def test_change_min_date(self):
        """ Test changing the minimum date.

        """
        component = self.component
        new_minimum = date(2000,1,1)
        component.min_date = new_minimum
        self.assertEnamlInSync(component, 'min_date', new_minimum)

    def test_change_max_date(self):
        """ Test changing the maximum date.

        """
        component = self.component
        new_maximum = date(2005,1,1)
        component.max_date = new_maximum
        self.assertEnamlInSync(component, 'max_date', new_maximum)

    def test_change_date_in_enaml(self):
        """ Test changing the current date through the component.

        """
        component = self.component
        new_date = date(2007,10,9)
        component.date = new_date
        self.assertEnamlInSync(component, 'date', new_date)
        self.assertEqual(self.events, [])

    def test_invalid_min_date(self):
        """ Test changing to an invalid date below the min range.

        """
        component = self.component
        min_date = date(2000,2,3)
        component.min_date = min_date
        with self.assertRaises(TraitError):
            component.date = date(2000,1,1)
        self.assertEnamlInSync(component, 'date', date.today())
        self.assertEqual(self.events, [])

    def test_invalid_max_date(self):
        """ Test changing to an invalid date above the max range.

        """
        component = self.component
        component.date = date(2011,10,9)
        self.assertEqual(self.events, [])
        max_date = date(2014,2,3)
        component.max_date = max_date
        with self.assertRaises(TraitError):
            component.date = date(2016,10,9)
        self.assertEnamlInSync(component, 'date', date(2011,10,9))
        self.assertEqual(self.events, [])

    def test_select_date_in_ui(self):
        """ Test changing the current date thought the ui

        """
        component = self.component
        widget = self.widget
        new_date = date(2007,10,9)
        self.select_date(widget, new_date)
        self.assertEqual(self.get_date(widget), new_date)
        # make sure that the component is not updated when selected is fired
        self.assertEqual(component.date, date.today())
        self.assertEqual(self.events, [('selected', new_date)])

    def test_activate_date_in_ui(self):
        """ Test activating the current date thought the ui

        """
        component = self.component
        widget = self.widget
        new_date = date(2007,10,9)
        self.select_date(widget, new_date)
        self.activate_date(self.widget, new_date)
        self.assertEnamlInSync(component, 'date', new_date)
        self.assertEqual(self.events, [('selected', new_date),
                                       ('activated', new_date)])

    def test_change_maximum_and_date(self):
        """ Test setting maximum while the date is out of range.

        """
        component = self.component
        component.date = date(2007,10,9)
        component.max_date = date(2006,5,9)
        self.assertEnamlInSync(component, 'date', date(2006,5,9))
        self.assertEqual(self.events, [])

    def test_change_minimum_and_date(self):
        """ Test setting minimum while the date is out of range.

        """
        component = self.component
        component.date = date(2007,10,9)
        component.min_date = date(2010,5,9)
        self.assertEnamlInSync(component, 'date', date(2010,5,9))
        self.assertEqual(self.events, [])

    def test_change_range_invalid(self):
        """ Test setting minimum > maximum.

        """
        component = self.component
        component.min_date = date(2010,5,9)
        with self.assertRaises(TraitError):
            component.max_date = date(2006,5,9)

        component.max_date = date(2034,12,10)
        with self.assertRaises(TraitError):
            component.min_date = date(2034,12,14)

    #--------------------------------------------------------------------------
    # Special initialization tests
    #--------------------------------------------------------------------------
    def test_initial_too_early(self):
        """ Check initialization with an invalid early date is corrected.

        .. todo:: avoid using the enaml source

        """
        enaml_source = """
import datetime
enamldef MainView(MainWindow):
    attr events
    Calendar:
        name = 'cal'
        date = datetime.date(1980, 1, 1)
        min_date = datetime.date(1990, 1, 1)
        max_date = datetime.date(2000, 1, 1)
        selected :: events.append(('selected', event.new))
        activated :: events.append(('activated', event.new))

"""
        events = []
        with self.assertRaises(TraitError):
            self.parse_and_create(enaml_source, events=events)

    def test_initial_too_late(self):
        """ Check initialization with an invalid late date is corrected.

        .. todo:: avoid using the enaml source

        """
        enaml = """
import datetime
enamldef MainView(MainWindow):
    attr events
    Calendar:
        name = 'cal'
        date = datetime.date(2010, 1, 1)
        min_date = datetime.date(1990, 1, 1)
        max_date = datetime.date(2000, 1, 1)
        selected :: events.append(('selected', event.new))
        activated :: events.append(('activated', event.new))
"""
        events = []
        with self.assertRaises(TraitError):
            self.parse_and_create(enaml, events=events)

    #--------------------------------------------------------------------------
    # Abstract methods
    #--------------------------------------------------------------------------
    @required_method
    def get_date(self, widget):
        """ Get a calendar's active date.

        """
        pass

    @required_method
    def get_min_date(self, widget):
        """ Get a calendar's minimum date attribute.

        """
        pass

    @required_method
    def get_max_date(self, widget):
        """ Get a calendar's maximum date attribute.

        """
        pass

    @required_method
    def activate_date(self, widget):
        """ Fire an event to indicate that a date was activated.

        """
        pass

    @required_method
    def select_date(self, widget, date):
        """ Fire an event to indicate that a date was selected.

        """
        pass




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .qt import QtGui
from .qt_widget_component import QtWidgetComponent

from ...components.menu import AbstractTkMenu


class QtMenu(QtWidgetComponent, AbstractTkMenu):
    """ A Qt4 implementation of Menu.

    """
    #--------------------------------------------------------------------------
    # Setup Methods
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Create the underlying QMenu object.

        """
        # We ignore the parent when creating the menu. The parent menu
        # is repsonsible for adding menus to itself, and using the parent
        # here causes issues on certain platforms.
        self.widget = QtGui.QMenu()

    def initialize(self):
        """ Initialize the underlying QMenu object.

        """
        super(QtMenu, self).initialize()
        self.set_title(self.shell_obj.title)
        self.update_contents()
    
    def bind(self):
        """ Binds the event handlers for the underlying QMenu object.

        """
        super(QtMenu, self).bind()
        widget = self.widget
        widget.aboutToShow.connect(self.on_about_to_show)
        widget.aboutToHide.connect(self.on_about_to_hide)

    #--------------------------------------------------------------------------
    # Change Handlers 
    #--------------------------------------------------------------------------
    def shell_title_changed(self, text):
        """ The change handler for the 'title' attribute on the shell
        object.

        """
        self.set_title(text)
        
    def shell_contents_changed(self, contents):
        """ The change handler for the 'contents' attribute on the shell
        object. 

        """
        self.update_contents()

    #--------------------------------------------------------------------------
    # Signal Handlers
    #--------------------------------------------------------------------------
    def on_about_to_show(self):
        """ A signal handler for the 'aboutToShow' signal of the QMenu.

        """
        self.shell_obj.about_to_show()

    def on_about_to_hide(self):
        """ A signal handler for the 'aboutToHide' signal of the QMenu.

        """
        self.shell_obj.about_to_hide()

    #--------------------------------------------------------------------------
    # Widget Update Methods 
    #--------------------------------------------------------------------------
    def update_contents(self):
        """ Updates the contents of the Menu.

        """
        # There is no need to clear() the Menu since that destroys the
        # underlying objects, and any dynamic children will have already
        # been destroyed. It's sufficient to just make a pass over the 
        # contents and add them to the menu. Qt is smart enough to do the
        # right thing here.
        widget = self.widget
        for item in self.shell_obj.contents:
            child_widget = item.toolkit_widget
            if isinstance(child_widget, QtGui.QMenu):
                widget.addMenu(child_widget)
            elif isinstance(child_widget, QtGui.QAction):
                widget.addAction(child_widget)
            else:
                msg = 'Invalid child for QMenu: %s'
                raise TypeError(msg % child_widget)

    def set_title(self, title):
        """ Sets the title of the QMenu object.

        """
        self.widget.setTitle(title)

    def set_visible(self, visible):
        """ Overridden parent class method which sets the visibility
        of the underlying action for the menu which provides the proper
        semantic behavior.

        """
        self.widget.menuAction().setVisible(visible)

    #--------------------------------------------------------------------------
    # Auxiliary Methods 
    #--------------------------------------------------------------------------
    def popup(self, blocking=True):
        """ Pops up the menu using a blocking context if requested,
        using the current position of the cursor.

        """
        pos = QtGui.QCursor.pos()
        if blocking:
            self.widget.exec_(pos)
        else:
            self.widget.popup(pos)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from numbers import Real, Integral, Complex

from .number_validators import NumberValidator


class NumberExpressionSandboxError(RuntimeError):
    """ A RuntimeError subclass indicating a sandbox access violation.

    """
    pass


class NumberExpressionSandbox(object):
    """ A simple sandbox for us with NumberValidator that does allow 
    name access.

    """
    allowed = {
        'int': int, 'float': float, 'long': long, 'complex': complex,
    }

    def __getitem__(self, name):
        allowed = self.allowed
        if name not in allowed:
            msg = "Expression sandbox does not allow access to '%s'" % name
            raise NumberExpressionSandboxError(msg)
        return allowed[name]

    def __setitem__(self, name, value):
        msg = "Expression sandbox does not allow writes"
        raise NumberExpressionSandboxError(msg)


class NumberExpressionValidator(NumberValidator):
    """ A NumberValidator subclass that accepts a string expression 
    which evaluates to a Python number. 

    The string evaluation is performed in a sandboxed environment which
    allows limited access to Python builtins. Locale information is not
    used for formatting or conversion.

    """
    #: The sandbox in which the input string will be evaluated. This 
    #: should typically not be manipulated by user code, lest ye 
    #: suffer the security vulnerabilities.
    sandbox = ({'__builtins__': {}}, NumberExpressionSandbox())

    def _convert(self, text):
        """ Overridden default convert method which safetly evaluates 
        the given string in a sandboxed environment. 

        It does not use the locale object to perform conversion since 
        it allows for generic arithmetic expressions which must make
        a successful round-trip.

        """
        return eval(text, *self.sandbox)
        
    def _coerce(self, value):
        """ Overriden default coerce method which attempts to use the
        number type to coerce the value if it is not already of the
        proper type.

        """
        if not isinstance(value, self.number_type):
            try:
                value = self.number_type(value)
            except (TypeError, ValueError):
                pass
        return value

    def validate(self, text):
        """ Overridden default validator which converts any INVALID
        result into INTERMEDIATE. This allows arbitrary expresssions
        to be entered into the field.

        """
        v = super(NumberExpressionValidator, self).validate(text)
        if v == self.INVALID:
            v = self.INTERMEDIATE
        return v


class IntegralNumberExpressionValidator(NumberExpressionValidator):
    """ A NumberExpressionValidator for Integral number types using the
    abstract base class numbers.Integral for type checking.

    """
    number_type = Integral


class RealNumberExpressionValidator(NumberExpressionValidator):
    """ A NumberExpressionValidator for Real number types using the 
    abstract base class numbers.Real for type checking.

    """
    number_type = Real


class ComplexNumberExpressionValidator(NumberExpressionValidator):
    """ A NumberExpressionValidator for Complex number types using the 
    abstract base class numbers.Complex for type checking.

    """
    number_type = Complex


class IntExpressionValidator(IntegralNumberExpressionValidator):
    """ A NumberExpressionValidator for 'int' number types.

    """
    number_type = int


class LongExpressionValidator(IntegralNumberExpressionValidator):
    """ A NumberExpressionValidator for 'long' number types.

    """
    number_type = long


class FloatExpressionValidator(RealNumberExpressionValidator):
    """ A NumberExpressionValidator for 'float' number types.

    """
    number_type = float


class ComplexExpressionValidator(ComplexNumberExpressionValidator):
    """ A NumberExpressionValidator for 'complex' number types.

    """
    number_type = complex


class BinExpressionValidator(IntExpressionValidator):
    """ An IntExpressionValidator which formats as a binary string.

    """
    def _format(self, value):
        """ Overridden default format method to format as a binary
        string.

        """
        return bin(value)


class OctExpressionValidator(IntExpressionValidator):
    """ An IntExpressionValidator which formats as an octal string.

    """
    def _format(self, value):
        """ Overridden default format method to format as an octal
        string.

        """
        return oct(value)


class HexExpressionValidator(IntExpressionValidator):
    """ An IntExpressionValidator which formats as a hex string.

    """
    def _format(self, value):
        """ Overridden default format method to format as a hex
        string.

        """
        return hex(value)


class LongBinExpressionValidator(LongExpressionValidator):
    """ A LongExpressionValidator which formats as a binary string.

    """
    def _format(self, value):
        """ Overridden default format method to format as a binary
        string.

        """
        return bin(value)


class LongOctExpressionValidator(LongExpressionValidator):
    """ A LongExpressionValidator which formats as an octal string.

    """
    def _format(self, value):
        """ Overridden default format method to format as an octal
        string.

        """
        return oct(value)


class LongHexExpressionValidator(LongExpressionValidator):
    """ A LongExpressionValidator which formats as a hex string.

    """
    def _format(self, value):
        """ Overridden default format method to format as a hex
        string.

        """
        return hex(value)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import wx

from .wx_control import WXControl

from ...components.progress_bar import AbstractTkProgressBar


class WXProgressBar(WXControl, AbstractTkProgressBar):
    """ WX implementation of ProgressBar.

    """
    #--------------------------------------------------------------------------
    # Setup Methods
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Creates the underlying wx.Gauge.

        """
        self.widget = wx.Gauge(parent)

    def initialize(self):
        """ Initialize the attributes of the progress bar.

        """
        super(WXControl, self).initialize()
        shell = self.shell_obj
        self.set_minimum(shell.minimum)
        self.set_maximum(shell.maximum)
        self.set_value(shell.value)

    #--------------------------------------------------------------------------
    # Abstract Implementation Methods
    #--------------------------------------------------------------------------
    def shell_value_changed(self, value):
        """ The change handler for the 'value' attribute of the shell
        object.

        """
        self.set_value(value)

    def shell_minimum_changed(self, minimum):
        """ The change handler for the 'minimum' attribute of the shell
        object.

        """
        self.set_minimum(minimum)
            
    def shell_maximum_changed(self, maximum):
        """ The change handler for the 'maximum' attribute of the shell
        object

        """
        self.set_maximum(maximum)

    #--------------------------------------------------------------------------
    # Widget Update Methods
    #--------------------------------------------------------------------------
    def set_value(self, value):
        """ Sets the value of the progress bar.

        """
        gauge_value = value - self.shell_obj.minimum
        self.widget.SetValue(gauge_value)

    def set_minimum(self, minimum):
        """ Sets the minimum value of the progress bar.

        """
        gauge_range = self.shell_obj.maximum - minimum
        self.widget.SetRange(gauge_range)
    
    def set_maximum(self, maximum):
        """ Sets the maximum value of the progress bar.

        """
        gauge_range = maximum - self.shell_obj.minimum
        self.widget.SetRange(gauge_range)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from datetime import datetime as python_datetime

from traits.api import TraitError

from .enaml_test_case import EnamlTestCase, required_method


class TestDatetimeEdit(EnamlTestCase):
    """ Logic for testing the date time edit components.

    Tooklit testcases need to provide the following functions

    Abstract Methods
    ----------------
    get_datetime(self, widget)
        Get the toolkits widget's active datetime.

    get_min_datetime(self, widget)
        Get the toolkits widget's maximum datetime attribute.

    get_max_datetime(self, widget)
        Get the toolkits widget's minimum datetime attribute.

    change_datetime(self, widget, date)
        Simulate a change datetime action at the toolkit widget.

    get_datetime_as_string(self, widget)
        Get the toolkits widget's active datetime as a string.

    """

    def setUp(self):
        """ Set up for the TimeEdit testcases

        """

        enaml_source = """
from datetime import datetime as python_datetime
enamldef MainView(MainWindow):
    attr events
    DatetimeEdit:
        name = 'test'
        datetime = python_datetime(2001, 4, 3, 8, 45, 32, 23000)
        datetime_changed :: events.append(('datetime_changed', event.new))
"""

        self.default_datetime = python_datetime(2001, 4, 3, 8, 45, 32, 23000)
        self.events = []
        self.view = self.parse_and_create(enaml_source, events=self.events)
        self.component = self.component_by_name(self.view, 'test')
        self.widget = self.component.toolkit_widget

    def test_initialization_values(self):
        """ Test the initial attributes of the date edit component.

        """
        component = self.component

        self.assertEnamlInSync(component, 'datetime',
                               python_datetime(2001, 4, 3, 8, 45, 32, 23000))
        self.assertEnamlInSync(component, 'min_datetime',
                               python_datetime(1752,9,14,0,0,0,0))
        self.assertEnamlInSync(component, 'max_datetime',
                               python_datetime(7999, 12, 31, 23, 59, 59, 999000))
        self.assertEqual(self.events, [])

    def test_change_max_datetime(self):
        """ Test changing the maximum datetime.

        """
        component = self.component
        new_maximum = python_datetime(2005,1,1)
        component.max_datetime = new_maximum
        self.assertEnamlInSync(component, 'max_datetime', new_maximum)
        self.assertEqual(self.events, [])

    def test_change_min_datetime(self):
        """ Test changing the minimum datetime.

        """
        component = self.component
        new_minimum = python_datetime(2000,1,1)
        component.min_datetime = new_minimum
        self.assertEnamlInSync(component, 'min_datetime', new_minimum)
        self.assertEqual(self.events, [])

    def test_change_maximum_and_datetime(self):
        """ Test setting maximum while the datetime is out of range.

        """
        component = self.component
        component.datetime = python_datetime(2007,10,9)
        component.max_datetime = python_datetime(2006,5,9)
        self.assertEnamlInSync(component, 'datetime', python_datetime(2006,5,9))
        self.assertEqual(self.events, [])

    def test_change_minimum_and_datetime(self):
        """ Test setting minimum while the datetime is out of range.

        """
        component = self.component
        component.datetime = python_datetime(2007,10,9)
        component.min_datetime = python_datetime(2010,5,9)
        self.assertEnamlInSync(component, 'datetime', python_datetime(2010,5,9))
        self.assertEqual(self.events, [])

    def test_change_datetime_in_enaml(self):
        """ Test changing the current datetime through the component.

        """
        component = self.component
        new_datetime = python_datetime(2007,10,9)
        component.datetime = new_datetime
        self.assertEnamlInSync(component, 'datetime', new_datetime)
        self.assertEqual(self.events, [])

    def test_change_datetime_in_ui(self):
        """ Test changing the current datetime thought the ui

        """
        component = self.component
        widget = self.widget
        new_datetime = python_datetime(2007,10,9)
        self.change_datetime(widget, new_datetime)
        self.assertEnamlInSync(component, 'datetime', new_datetime)
        self.assertEqual(self.events, [('datetime_changed', new_datetime)])

    def test_invalid_min_datetime(self):
        """ Test changing to an invalid datetime below the min range.

        """
        component = self.component
        min_datetime = python_datetime(2000,2,3)
        component.min_datetime = min_datetime
        with self.assertRaises(TraitError):
            component.datetime = python_datetime(2000,1,1)
        self.assertEnamlInSync(component, 'datetime', self.default_datetime)

    def test_invalid_max_datetime(self):
        """ Test changing to an invalid datetime above the max range.

        """
        component = self.component
        init_datetime = python_datetime(2011,10,9)
        component.datetime = init_datetime
        self.assertEqual(self.events, [])
        max_datetime = python_datetime(2014,2,3)
        component.max_datetime = max_datetime
        with self.assertRaises(TraitError):
            component.datetime = python_datetime(2016,10,9)
        self.assertEnamlInSync(component, 'datetime', init_datetime)
        self.assertEqual(self.events, [])

    def test_set_format(self):
        """ Test setting the output format

        """
        component = self.component
        widget = self.widget
        component.datetime_format = 'MMM dd yyyy hh:mm'
        test_datetime = python_datetime(2007,10,9, 2, 34, 12,2000)
        component.datetime = test_datetime
        widget_string = self.get_datetime_as_string(widget)
        formated_date = unicode(test_datetime.strftime('%b %d %Y %H:%M'), encoding='utf-8')
        self.assertEqual(widget_string, formated_date)
        self.assertEqual(self.events, [])

    def test_change_range_invalid(self):
        """ Test setting minimum > maximum.

        """
        component = self.component
        component.min_datetime = python_datetime(2010,5,9)
        with self.assertRaises(TraitError):
            component.max_datetime = python_datetime(2006,5,9)

        component.max_datetime = python_datetime(2034,12,10)
        with self.assertRaises(TraitError):
            component.min_date = python_datetime(2034,12,14)

    #--------------------------------------------------------------------------
    # Special initialization tests
    #--------------------------------------------------------------------------
    def test_initial_too_early(self):
        """ Check initialization with an invalid early datetime is corrected.

        .. todo:: avoid using the enaml source

        """
        enaml_source = """
from datetime import datetime as python_datetime
enamldef MainView(MainWindow):
    attr events
    DatetimeEdit:
        name = 'test'
        datetime = python_datetime(1980, 1, 1, 23, 10, 34)
        min_datetime = python_datetime(1990, 1, 1)
        max_datetime = python_datetime(2000, 1, 1)
        datetime_changed :: events.append(('datetime_changed', event.new))
"""
        events = []
        with self.assertRaises(TraitError):
            self.parse_and_create(enaml_source, events=events)

    def test_initial_too_late(self):
        """ Check initialization with an invalid late datetime is corrected.

        .. todo:: avoid using the enaml source

        """
        enaml_source = """
from datetime import datetime as python_datetime
enamldef MainView(MainWindow):
    attr events
    DatetimeEdit:
        name = 'test'
        datetime = python_datetime(2010, 1, 1, 9, 12, 34, 14234)
        min_datetime = python_datetime(1990, 1, 1)
        max_datetime = python_datetime(2000, 1, 1)
        datetime_changed :: events.append(('datetime_changed', event.new))
"""
        events = []
        with self.assertRaises(TraitError):
            self.parse_and_create(enaml_source, events=events)

    #--------------------------------------------------------------------------
    # Abstract methods
    #--------------------------------------------------------------------------
    @required_method
    def get_datetime(self, widget):
        """  Get the toolkits widget's active datetime.

        """
        pass

    @required_method
    def get_min_datetime(self, widget):
        """  Get the toolkits widget's maximum datetime attribute.

        """
        pass

    @required_method
    def get_max_datetime(self, widget):
        """ Get the toolkits widget's minimum datetime attribute.

        """
        pass

    @required_method
    def change_datetime(self, widget, date):
        """ Simulate a change datetime action at the toolkit widget.

        """
        pass

    @required_method
    def get_datetime_as_string(self, widget):
        """  Get the toolkits widget's active datetime as a string.

        """
        pass




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from ...core.constructor import Constructor


def importer(module_path, name):
    def _importer():
        mod = __import__(module_path, fromlist=[name])
        try:
            res = getattr(mod, name)
        except AttributeError:
            raise ImportError('Cannot import name %s' % name)
        return res
    return _importer


def constructor(base_path):
    """ A factory function which understands our name mangling and will
    create a constructor instance. Returns tuple of (name, ctor) where
    name is a string that can be used by toolkit to refer to the ctor
    in the enaml source code.

    """
    c_module_path = 'enaml.components.' + base_path
    c_name = ''.join(part.capitalize() for part in base_path.split('_'))

    t_module_path = 'enaml.backends.wx.' + 'wx_' + base_path
    t_name = 'WX' + c_name

    component_loader = importer(c_module_path, c_name)
    abstract_loader = importer(t_module_path, t_name)

    ctor = Constructor(component_loader, abstract_loader)

    return c_name, ctor


def tui_imp():
    from ...components.traitsui_item import TraitsUIItem
    return TraitsUIItem


def wx_tui_imp():
    from .wx_traitsui_item import WXTraitsUIItem
    return WXTraitsUIItem


tui_item = Constructor(tui_imp, wx_tui_imp)


WX_CONSTRUCTORS = dict((
    constructor('window'),
    constructor('main_window'),
    constructor('dialog'),
    constructor('directory_dialog'),
    constructor('container'),
    constructor('calendar'),
    constructor('check_box'),
    constructor('combo_box'),
    constructor('field'),
    constructor('file_dialog'),
    constructor('html'),
    constructor('image_view'),
    constructor('label'),
    constructor('push_button'),
    constructor('radio_button'),
    constructor('toggle_button'),
    constructor('slider'),
    constructor('spin_box'),
    ('TraitsUIItem', tui_item),
    constructor('enable_canvas'),
    constructor('table_view'),
    constructor('date_edit'),
    constructor('datetime_edit'),
    constructor('form'),
    constructor('group_box'),
    constructor('progress_bar'),
    constructor('scroll_area'),
    constructor('tab_group'),
    constructor('tab'),
    constructor('splitter'),
    constructor('float_slider'),
    constructor('menu_bar'),
    constructor('menu'),
    constructor('action'),
))




#------------------------------------------------------------------------------
#  Copyright (c) 2012, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from abc import ABCMeta, abstractmethod, abstractproperty

from ..utils import abstractclassmethod


class AbstractTkImage(object):
    """ An abstract base class which represents a toolkit independent
    abstraction of a raster image.
    
    """
    __metaclass__ = ABCMeta

    @abstractclassmethod
    def from_data(cls, data, size):
        """ Construct an image from a bytearray of RGBA bytes.

        Parameters
        ----------
        data : bytearray
            A bytearray of the image data in RGBA32 format.

        size : (width, height)
            The width, height size tuple of the image.

        Returns
        -------
        results : AbstractTkImage
            An appropriate image instance.

        """
        raise NotImplementedError
    
    @abstractclassmethod
    def from_file(cls, path, format=''):
        """ Read in the image data from a file.

        Parameters
        ----------
        path : string
            The path to the image file on disk.

        format : string, optional
            The image format of the data in the file. If not given,
            then the image format will be determined automatically
            if possible.
        
        Returns
        -------
        results : AbstractTkImage
            An appropriate image instance.

        """
        raise NotImplementedError

    @abstractproperty
    def size(self):
        """ The size of the image as a (width, height) tuple.
        
        Returns
        -------
        result : (width, height)
            The width, height size tuple of the image.

        """
        raise NotImplementedError
    
    @abstractmethod
    def data(self):
        """ The data for the image as a bytearray of RGBA bytes.
        
        Returns
        -------
        results : bytearray
            A bytearray of the image data in RGBA32 format.

        """
        raise NotImplementedError

    @abstractmethod
    def scale(self, size):
        """ Create a new version of this image scaled to the given size.
        
        Parameters
        ----------
        size : (width, height)
            The size of the scaled image.
            
        Returns
        -------
        results : AbstractTkImage
            A new image of the proper scaled size.
            
        """
        raise NotImplementedError




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
""" A simple module that acts like a remote db for stock info. """
import time
import datetime
import numpy as np


def datestr2ts(date):
    year, month, day = date.split('-')
    dt = datetime.date(int(year), int(month), int(day))
    return time.mktime(dt.timetuple())


class TickData(object):

    usecols = (2, 3, 4, 5, 6, 7)

    converters = {
        2: datestr2ts,
        3: float,
        4: float,
        5: float,
        6: float,
        7: int,
    }

    dtype = np.dtype([
        ('dates', 'float'), 
        ('open', 'float'), 
        ('high', 'float'),
        ('low', 'float'), 
        ('close', 'float'), 
        ('volume', 'int'),
    ])

    def __init__(self, path):
        self._arr = np.loadtxt(path, dtype=self.dtype, usecols=self.usecols,
                               delimiter=',', converters=self.converters)[::-1]
        
    @property
    def min_date(self):
        return self._arr['dates'][0]
    
    @property
    def max_date(self):
        return self._arr['dates'][-1]
    
    def get_data(self, start, end, numpoints):
        arr = self._arr
        dates = np.linspace(start, end, numpoints)
        out = np.empty((numpoints,), dtype=self.dtype)
        out['dates'] = dates
        indexes = arr['dates']
        out['open'] = np.interp(dates, indexes, arr['open'])
        out['close'] = np.interp(dates, indexes, arr['close'])
        out['low'] = np.interp(dates, indexes, arr['low'])
        out['high'] = np.interp(dates, indexes, arr['high'])
        out['volume'] = np.interp(dates, indexes, arr['volume'])
        return out


_db_symbols = {
    'MSFT': TickData('./demo_support/msft.csv'),
    'AAPL': TickData('./demo_support/aapl.csv'),
    'GOOG': TickData('./demo_support/goog.csv'),
}


def get_symbols():
    return sorted(_db_symbols.keys())


def get_symbol_date_range(symbol):
    tick_data = _db_symbols.get(symbol)
    if not tick_data:
        return
    return (datetime.date.fromtimestamp(tick_data.min_date), 
            datetime.date.fromtimestamp(tick_data.max_date))


def get_data(symbol, start_date, end_date, numpoints):
    tick_data = _db_symbols.get(symbol)
    if not tick_data:
        return
    startts = time.mktime(start_date.timetuple())
    endts = time.mktime(end_date.timetuple())
    return tick_data.get_data(startts, endts, numpoints)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import wx

from .wx_control import WXControl

from ...components.slider import AbstractTkSlider


#: A map from Enaml constants to wxSlider TickPosition values for a
#: horizontal slider.
HORIZ_TICK_POS_MAP = {'top': wx.SL_TOP, 
                      'bottom': wx.SL_BOTTOM,
                      'both': wx.SL_BOTH}


#: A map from Enaml constants to wxSlider TickPosition values for a
#: vertical slider.
VERT_TICK_POS_MAP = {'left': wx.SL_LEFT, 
                     'right': wx.SL_RIGHT,
                     'both': wx.SL_BOTH}


#: A map that adapts horizontal positions to vertical positions
ADAPT_HORIZ_TICK = {'left': 'top', 'right': 'bottom'}


#: A map that adapts vertical positions to horizontal positions
ADAPT_VERT_TICK = {'top': 'left', 'bottom': 'right'}


class WXSlider(WXControl, AbstractTkSlider):
    """ A wxPython implementation of Slider.

    """
    #--------------------------------------------------------------------------
    # Setup Methods
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Creates the underlying wx.Slider control.

        """
        self.widget = wx.Slider(parent=parent)

    def initialize(self):
        """ Initializes the attributes of the toolkit widget.

        """
        super(WXSlider, self).initialize()
        # Setting double buffered reduces flicker on Windows
        self.widget.SetDoubleBuffered(True)
        shell = self.shell_obj
        shell._down = False
        self.set_range(shell.minimum, shell.maximum)
        self.set_position(shell.value)
        self.set_orientation(shell.orientation)
        self.set_tick_position(shell.tick_position)
        self.set_tick_frequency(shell.tick_interval)
        self.set_single_step(shell.single_step)
        self.set_page_step(shell.page_step)

    def bind(self):
        """ Binds the event handlers for the slider widget.

        """
        super(WXSlider, self).bind()
        widget = self.widget
        widget.Bind(wx.EVT_SCROLL_THUMBTRACK, self._on_slider_moved)
        widget.Bind(wx.EVT_SCROLL_TOP, self._on_slider_keyed)
        widget.Bind(wx.EVT_SCROLL_BOTTOM, self._on_slider_keyed)
        widget.Bind(wx.EVT_SCROLL_LINEUP, self._on_slider_keyed)
        widget.Bind(wx.EVT_SCROLL_LINEDOWN, self._on_slider_keyed)
        widget.Bind(wx.EVT_SCROLL_PAGEUP, self._on_slider_keyed)
        widget.Bind(wx.EVT_SCROLL_PAGEDOWN, self._on_slider_keyed)
        widget.Bind(wx.EVT_LEFT_DOWN, self._on_left_down)
        widget.Bind(wx.EVT_LEFT_UP, self._on_thumb_released)

    #--------------------------------------------------------------------------
    # Implementation
    #--------------------------------------------------------------------------
    def shell_minimum_changed(self, minimum):
        """ The change handler for the 'minimum' attribute on the shell
        object.

        """
        self.set_range(minimum, self.shell_obj.maximum)

    def shell_maximum_changed(self, maximum):
        """ The change handler for the 'maximum' attribute on the shell
        object.
        
        """
        self.set_range(self.shell_obj.minimum, maximum)

    def shell_value_changed(self, value):
        """ The change handler for the 'value' attribute on the shell
        object.
        
        """
        self.set_position(value)

    def shell_tracking_changed(self, tracking):
        """ The change handler for the 'tracking' attribute on the shell
        object.

        """
        # wx doesn't support tracking modes, so we fake it when we 
        # receive a scroll event.
        pass

    def shell_single_step_changed(self, single_step):
        """ The change handler for the 'single_step' attribute on the 
        shell object.
        
        """        
        self.set_single_step(single_step)

    def shell_page_step_changed(self, page_step):
        """ The change handler for the 'page_step' attribute on the 
        shell object.

        """
        self.set_page_step(page_step)

    def shell_tick_interval_changed(self, tick_interval):
        """ The change handler for the 'tick_interval' attribute on the
        shell object.

        """
        shell = self.shell_obj
        self.set_tick_frequency(tick_interval)
        # This extra calls are made since the range trait on 
        # shell object may clip the values to fit within the 
        # range without firing a changed notification.
        self.set_single_step(shell.single_step)
        self.set_page_step(shell.page_step)

    def shell_tick_position_changed(self, tick_position):
        """ The change handler for the 'tick_position' attribute on the
        shell object.

        """
        self.set_tick_position(tick_position)
        self.shell_obj.size_hint_updated()

    def shell_orientation_changed(self, orientation):
        """ The change handler for the 'orientation' attribute on the 
        shell object.

        """
        self.set_orientation(orientation)
        self.shell_obj.size_hint_updated()

    #--------------------------------------------------------------------------
    # Event Handlers 
    #--------------------------------------------------------------------------
    def _on_slider_moved(self, event):
        """ The event handler for a slider drag event.

        """
        shell = self.shell_obj
        value = self.widget.GetValue()

        # Update the value only if tracking is turned on.
        if shell.tracking:
            shell.value = value
        
        # Since this event is only fired when the drag handle is 
        # moved with the mouse, it's safe to fire the move event
        # with no further checks.
        shell.moved(value)

        event.Skip()

    def _on_slider_keyed(self, event):
        """ The event handler for the user changing the value of the 
        slider through a key.

        """
        self.shell_obj.value = self.widget.GetValue()
        event.Skip()

    def _on_left_down(self, event):
        """ The event handler for the left mouse button being pressed.
        This estimates the position of the drag handle and then checks 
        if the mouse was pressed over it and then fires the `pressed` 
        event and sets the `down` attribute.

        """
        # Wx does not provide an event for detecting a mouse press
        # on the drag handle. So we have to estimate it by doing a
        # bit of geometry. This is not very accurate but probably
        # sufficient for most use cases.
        shell = self.shell_obj
        widget = self.widget
        mouse_position = event.GetPosition()
        slider_position = widget.GetValue()
        thumb = widget.GetThumbLength() / 2.0
        width, height = widget.GetClientSizeTuple()

        if widget.HasFlag(wx.SL_VERTICAL):
            position = mouse_position[1] / float(height)
        else:
            position = mouse_position[0] / float(width)

        slider_length = float(widget.GetMax() - widget.GetMin()) + 1.0
        minimum = (slider_position - thumb) / slider_length
        maximum = (slider_position + thumb) / slider_length

        hit = minimum <= position <= maximum

        if hit:
            shell._down = True
            shell.pressed()
        
        event.Skip()

    def _on_thumb_released(self, event):
        """ The event handler for drag handle being released. It
        updates the value in the shell and fires the appropriate
        events.

        """
        shell = self.shell_obj
        value = self.widget.GetValue()
        shell.value = value
        if shell._down:
            shell._down = False
            shell.released()
        event.Skip()

    #--------------------------------------------------------------------------
    # Widget Update Methods
    #--------------------------------------------------------------------------
    def set_single_step(self, step):
        """ Set the single step attribute in the wx widget.

        """
        self.widget.SetLineSize(step)

    def set_page_step(self, step):
        """ Set the page step attribute in the wx widget.

        """
        self.widget.SetPageSize(step)

    def set_tick_position(self, ticks):
        """ Apply the tick position in the widget.

        """
        shell = self.shell_obj
        widget = self.widget
        style = widget.GetWindowStyle()
        style &= ~(wx.SL_TOP | wx.SL_BOTTOM | wx.SL_LEFT | wx.SL_RIGHT | 
                   wx.SL_BOTH | wx.SL_AUTOTICKS | wx.SL_TICKS)

        if shell.orientation == 'vertical':
            if ticks in ADAPT_VERT_TICK:
                shell.tick_position = ADAPT_VERT_TICK[ticks]
                return
            if ticks in VERT_TICK_POS_MAP:
                style |= (VERT_TICK_POS_MAP[ticks] | wx.SL_AUTOTICKS)
        else:
            if ticks in ADAPT_HORIZ_TICK:
                shell.tick_position = ADAPT_HORIZ_TICK[ticks]
                return
            if ticks in HORIZ_TICK_POS_MAP:
                style |= (HORIZ_TICK_POS_MAP[ticks] | wx.SL_AUTOTICKS)

        widget.SetWindowStyle(style)
        
        # There seems to be a bug in wx where the tick interval needs 
        # to be applied again for it to appear properly
        widget.SetTickFreq(shell.tick_interval)

    def set_orientation(self, orientation):
        """ Set the slider orientation.

        """
        widget = self.widget
        shell = self.shell_obj

        style = widget.GetWindowStyle()
        style &= ~(wx.SL_HORIZONTAL | wx.SL_VERTICAL)

        if orientation == 'vertical':
            style |= wx.SL_VERTICAL
        else:
            style |= wx.SL_HORIZONTAL

        widget.SetWindowStyle(style)
        self.set_tick_position(shell.tick_position)

    def set_range(self, minimum, maximum):
        """ Set the slider widget range.

        """
        self.widget.SetRange(minimum, maximum)

    def set_tick_frequency(self, interval):
        """ Set the slider widget tick mark fequency.

        """
        self.widget.SetTickFreq(interval)

    def set_position(self, value):
        """ set the position value.

        """
        self.widget.SetValue(value)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
""" An amalgamation of utilities used throughout the Enaml framework.

"""
class abstractclassmethod(classmethod):
    """ A backport of the Python 3's abc.abstractclassmethod.

    """
    __isabstractmethod__ = True

    def __init__(self, func):
        func.__isabstractmethod__ = True
        super(abstractclassmethod, self).__init__(func)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from abc import abstractmethod

from traits.api import Str, Instance

from .bounded_datetime import BoundedDatetime, AbstractTkBoundedDatetime

from ..core.trait_types import EnamlEvent


class AbstractTkDatetimeEdit(AbstractTkBoundedDatetime):
    """ The abstract toolkit DatetimeEdit interface.

    """
    @abstractmethod
    def shell_datetime_format_changed(self, datetime_format):
        """ The change handler for the 'datetime_format' attribute
        on the shell object.

        """
        raise NotImplementedError


class DatetimeEdit(BoundedDatetime):
    """ A datetime widget that displays a Python datetime.datetime object 
    using an appropriate toolkit specific control.

    """
    #: A python date format string to format the datetime. If None is
    #: supplied (or is invalid) the system locale setting is used.
    #: This may not be supported by all backends.
    datetime_format = Str

    #: Triggered whenever the user changes the date through the ui
    #: control, but not programmatically. The event payload will be 
    #: the datetime on the control.
    datetime_changed = EnamlEvent
    
    #: How strongly a component hugs its contents' width. DatetimeEdits 
    #: ignore the width hug by default, so they expand freely in width.
    hug_width = 'ignore'

    #: Overridden parent class trait
    abstract_obj = Instance(AbstractTkDatetimeEdit)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .qt import QtGui, QtCore
from .qt_window import QtWindow, QtWindowLayout
from .qt_resizing_widgets import QResizingDialog

from ...components.dialog import AbstractTkDialog


_MODAL_MAP = {
    'application_modal': QtCore.Qt.ApplicationModal,
    'window_modal': QtCore.Qt.WindowModal,
}


class QtDialog(QtWindow, AbstractTkDialog):
    """ A Qt implementation of a Dialog.

    This class creates a simple top-level dialog.

    """
    #--------------------------------------------------------------------------
    # Setup methods
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Creates the underlying QDialog control.

        """
        self.widget = QResizingDialog(parent)
        self.widget.setLayout(QtWindowLayout())

    def bind(self):
        """ Bind the signal handlers for the dialog.

        """
        super(QtDialog, self).initialize()
        self.widget.finished.connect(self._on_close)

    #--------------------------------------------------------------------------
    # Implementation
    #--------------------------------------------------------------------------
    def accept(self):
        """ Accept and close the dialog, sending the 'finished' signal.

        """
        self.widget.accept()

    def reject(self):
        """ Reject and close the dialog, sending the 'finished' signal.

        """
        self.widget.reject()

    #--------------------------------------------------------------------------
    # Event Handlers 
    #--------------------------------------------------------------------------
    def _on_close(self, qt_result):
        """ The event handler for the dialog's finished signal. 

        This translates from a QDialog result into an Enaml result enum
        value. The default result is rejection.

        """
        if qt_result == QtGui.QDialog.Accepted:
            result = 'accepted'
        else:
            result = 'rejected'
        shell = self.shell_obj
        shell._result = result
        shell._active = False
        shell.closed(result)

    #--------------------------------------------------------------------------
    # Overrides
    #--------------------------------------------------------------------------
    def set_visible(self, visible):
        """ Overridden from the parent class to properly launch and close 
        the dialog with the specified modality.

        """
        widget = self.widget
        shell = self.shell_obj
        if visible:
            shell._active = True
            shell.opened()
            widget.setWindowModality(_MODAL_MAP[shell.modality])
            widget.exec_()
        else:
            self.reject()




from traits.api import HasTraits, Str, Int
import enaml

with enaml.imports():
    from person_example import PersonWindow

class Person(HasTraits):
    name = Str
    age = Int
    address = Str

p = Person(name='Bob', age=18)
view = PersonWindow(p)
view.show()



#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .enaml_test_case import EnamlTestCase, required_method


class TestCheckBox(EnamlTestCase):
    """ Logic for testing push buttons.

    Tooklit testcases need to provide the following functions

    Abstract Methods
    ----------------
    get_text(widget)
        Returns the text from the tookit widget.

    checked_status(widget)
        Returns the checked status of the toolkit widget.

    Checkbox_pressed(self)
        Press the checkbox programmatically.

    Checkbox_released(self)
        Release the checkbox programmatically.

    Checkbox_toggled(self)
        Toggle the button programmatically.

    """
    def setUp(self):
        """ Setup enaml component for testing

        """
        self.check_box_label = 'checkbox label'

        enaml_source = """
enamldef MainView(MainWindow):
    attr events
    CheckBox:
        name = 'checkb1'
        text = 'checkbox label'
        checked = True
        toggled :: events.append('toggled')
        pressed :: events.append('pressed')
        released :: events.append('released')
""".format(self.check_box_label)

        self.events = []
        self.view = self.parse_and_create(enaml_source, events=self.events)
        self.component = self.component_by_name(self.view, 'checkb1')
        self.widget = self.component.toolkit_widget

    def test_box_initialization(self):
        """ Test the initialization of the widget

        """
        component = self.component
        # checked
        self.assertEnamlInSync(component, 'checked', True)
        self.assertEnamlInSync(component, 'text', self.check_box_label)

    def testLabelChange(self):
        """ Test changing the label of a check box

        """
        component = self.component
        new_label = 'new_label'
        component.text = new_label
        self.assertEnamlInSync(component, 'text', new_label)

    def testSettingChecked(self):
        """ Test selecting a WXCheckBox

        """
        component = self.component
        # un-check
        self.component.checked = False
        self.assertEnamlInSync(component, 'checked', False)
        # check
        self.component.checked = True
        self.assertEnamlInSync(component, 'checked', True)

    def test_checkbox_pressed(self):
        """ React to a checkbox press event.

        """
        events = self.events
        self.checkbox_pressed(self.widget)
        self.assertTrue(self.component.down)
        self.assertEqual(events, ['pressed'])

    def test_checkbox_toggled(self):
        """ Test a checkbox toggled event.

        """
        events = self.events
        self.checkbox_toggle(self.widget)
        self.assertFalse(self.component.down)
        self.assertEqual(events, ['toggled'])

    def test_checkbox_released(self):
        """ Test a checkbox release event.

        """
        events = self.events
        # Release events are ignored if the button was not already down
        self.checkbox_released(self.widget)
        self.assertEqual(events, [])

    def test_press_release_sequence(self):
        """ Verify the even firing when the press-release (nornal) 
        sequence is applied.

        """
        component = self.component
        widget = self.widget
        events = self.events
        self.checkbox_pressed(widget)
        self.assertTrue(component.down)
        self.checkbox_released(widget)
        self.assertFalse(component.down)
        self.assertEqual(events, ['pressed', 'released'])

    def test_checkbox_all_events(self):
        """ Test press, release, and click events.

        """
        self.checkbox_pressed(self.widget)
        self.checkbox_released(self.widget)
        self.checkbox_toggle(self.widget)

        events = self.events
        self.assertEqual(events, ['pressed', 'released', 'toggled'])

    #--------------------------------------------------------------------------
    # Abstract methods
    #--------------------------------------------------------------------------
    @required_method
    def get_text(self, widget):
        """ Returns the text from the tookit widget.

        """
        pass

    @required_method
    def get_checked(self, widget):
        """ Returns the checked status of the toolkit widget.

        """
        pass

    @required_method
    def checkbox_pressed(self, widget):
        """ Press the checkbox programmatically.

        """
        pass

    @required_method
    def checkbox_released(self, widget):
        """ Release the button programmatically.

        """
        pass

    @required_method
    def checkbox_toggle(self, widget):
        """ Toggle the button programmatically.

        """
        pass




#------------------------------------------------------------------------------
# Copyright (c) 2011, Enthought, Inc.
# All rights reserved.
#------------------------------------------------------------------------------
from abc import abstractmethod
from itertools import izip_longest

from traits.api import (
    Bool, List, Either, Int, Property, Instance, cached_property,
    on_trait_change,
)

from .constraints_widget import (
    ConstraintsWidget, AbstractTkConstraintsWidget,
)
from .container import Container
from .layout_task_handler import LayoutTaskHandler
from .widget_component import WidgetComponent

from ..enums import Orientation


class AbstractTkSplitter(AbstractTkConstraintsWidget):
    """ The abstract toolkit Splitter interface.

    A toolkit splitter container is responsible for handling changes on 
    a shell Splitter and proxying those changes to and from its internal 
    toolkit widget.

    """
    @abstractmethod
    def shell_splitter_children_changed(self, children):
        """ The change handler for the 'splitter_children' attribute of 
        the shell object.

        """
        raise NotImplementedError

    @abstractmethod
    def set_splitter_sizes(self, sizes):
        """ Set the splitter sizes for the children.

        """
        raise NotImplementedError

    @abstractmethod
    def shell_orientation_changed(self, orientation):
        """ The change handler for the 'orientation' attribute of the 
        shell object.

        """
        raise NotImplementedError

    @abstractmethod
    def shell_live_drag_changed(self, live_drag):
        """ The change handler for the 'live_drag' attribute of the
        shell object.

        """
        raise NotImplementedError
        

class Splitter(LayoutTaskHandler, ConstraintsWidget):
    """ A LayoutComponent subclass that displays its children in separate
    resizable compartements that are connected with a resizing bar.
     
    """
    #: The read-only list of widget children for the splitter.
    splitter_children = Property(
        List(Instance(WidgetComponent)), depends_on='children',
    )

    #: The orientation of the Splitter. 'horizontal' means the children 
    #: are laid out left to right, 'vertical' means top to bottom.
    orientation = Orientation('horizontal')

    #: Whether the child widgets resize as a splitter is being dragged
    #: (True), or if a simple indicator is drawn until the drag handle
    #: is released (False). The default is True.
    live_drag = Bool(True)
    
    #: A list of preferred sizes for each compartment of the splitter, or 
    #: None if there is no preference for the size.
    preferred_sizes = List(Either(None, Int, default=None))
    
    #: How strongly a component hugs it's contents' width. A Splitter
    #: container ignores its width hug by default, so it expands freely
    #: in width.
    hug_width = 'ignore'

    #: How strongly a component hugs it's contents' height. A Splitter
    #: container ignores its height hug by default, so it expands freely
    #: in height.
    hug_height = 'ignore'

    #: Overridden parent class trait
    abstract_obj = Instance(AbstractTkSplitter)
    
    #--------------------------------------------------------------------------
    # Property Getters
    #--------------------------------------------------------------------------
    @cached_property
    def _get_splitter_children(self):
        """ The cached property getter for the 'splitter_children' 
        attribute.

        """
        flt = lambda child: isinstance(child, WidgetComponent)
        return filter(flt, self.children)
    
    #--------------------------------------------------------------------------
    # Change Handlers
    #--------------------------------------------------------------------------
    @on_trait_change('splitter_children, orientation')
    def _on_splitter_deps_changed(self):
        """ The change handler which requests a relayout when the
        splitter children or orientation changes.

        """
        # XXX do we want to list to preferred_sizes as well?
        if self.initialized:
            self.request_relayout()

    #--------------------------------------------------------------------------
    # Setup Methods
    #--------------------------------------------------------------------------
    def _setup_finalize(self):
        """ Overridden setup method to initialize the sizes of the 
        splitter.

        """
        super(Splitter, self)._setup_finalize()
        self.update_splitter_sizes()

    #--------------------------------------------------------------------------
    # Overrides
    #--------------------------------------------------------------------------
    def do_relayout(self):
        """ A reimplemented LayoutTaskHandler handler method which will
        perform necessary update activity when a relayout it requested.

        """
        # This method is called whenever a relayout is requested.
        # We update the size of the splitter components and fire 
        # off a size hint updated event so that any parents can
        # react to our potential new size.
        self.update_splitter_sizes()
        self.size_hint_updated()

    #--------------------------------------------------------------------------
    # Update Methods
    #--------------------------------------------------------------------------
    def update_splitter_sizes(self):
        """ Update the sizes of each of the splitters based on the size
        hints of the children and the orientation of the splitter. The
        minimum sizes of the children are also computed and applied.

        """
        # TODO - the setting of the min sizes here could be cleaner
        num_children = len(self.splitter_children)
        sizes = []
        i = ['horizontal', 'vertical'].index(self.orientation)
        items = izip_longest(
            self.splitter_children, self.preferred_sizes[:num_children],
        )
        for child, size in items:
            if size is None:
                if isinstance(child, Container):
                    min_size = child.compute_min_size()
                    child.set_min_size(min_size)
                hint = child.size_hint()[i]
                if hint <= 0:
                    hint = child.min_size()[i]
                sizes.append(hint)
            else:
                sizes.append(size)
        self.abstract_obj.set_splitter_sizes(sizes)
    



#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from abc import abstractmethod

from traits.api import Int, Bool, Range, Instance

from .control import Control, AbstractTkControl

from ..validation import AbstractValidator, IntValidator


class AbstractTkSpinBox(AbstractTkControl):
    """ The abstract toolkit interface for a spin box.

    """
    @abstractmethod
    def shell_low_changed(self, low):
        raise NotImplementedError

    @abstractmethod
    def shell_high_changed(self, high):
        raise NotImplementedError

    @abstractmethod
    def shell_step_changed(self, step):
        raise NotImplementedError

    @abstractmethod
    def shell_value_changed(self, value):
        raise NotImplementedError

    @abstractmethod
    def shell_validator_changed(self, validator):
        raise NotImplementedError

    @abstractmethod
    def shell_wrap_changed(self, wrap):
        raise NotImplementedError
    
    @abstractmethod
    def shell_tracking_changed(self, tracking):
        raise NotImplementedError


class SpinBox(Control):
    """ A spin box widget which manipulates integer values.

    """
    #: The minimum value for the spin box. Defaults to 0.
    low = Int(0)

    #: The maximum value for the spin box. Defaults to 100.
    high = Int(100)

    #: The step size for the spin box. Defaults to 1.
    step = Int(1)

    #: The current integer value for the spin box, constrained to
    #: low <= value <= high.
    value = Range('low', 'high')

    #: A validator object to convert to and from a spin box integer
    #: and a unicode string for display. The format method will be 
    #: called with an integer and should return a string for display. 
    #: The convert method will be passed a string and should return 
    #: an int or raise a ValueError if the string cannot be converted. 
    #: If the conversion is succesful but the returned int does not fall
    #: within the allowed range of the spin box, then the spin box will
    #: not be updated. The default validator is a simple IntValidator.
    validator = Instance(AbstractValidator, factory=IntValidator)

    #: Whether or not the spin box will wrap around at its extremes. 
    #: Defaults to False.
    wrap = Bool(False)
    
    #: Whether the spin box will update on every key press (True), or
    #: only when enter is pressed or the widget loses focus (False).
    #: Defaults to False.
    tracking = Bool(False)

    #: How strongly a component hugs it's contents' width. SpinBoxes 
    #: ignore the width hug by default, so they expand freely in width.
    hug_width = 'ignore'

    #: Overridden parent class trait
    abstract_obj = Instance(AbstractTkSpinBox)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from abc import abstractmethod

from traits.api import (
    Bool, Int, Unicode, Enum, Instance, Any, List, on_trait_change,
)

from .control import Control, AbstractTkControl

from ..core.trait_types import EnamlEvent
from ..guard import guard
from ..validation import AbstractValidator, CoercingValidator


class AbstractTkField(AbstractTkControl):
    """ The abstract toolkit interface for a Field.

    Toolkit implementations of Field are slighly more complicated than
    other controls, given the flexibility required for validation and
    conversion.

    The major deviation from other toolkit controls is that the shell
    object is responsible for updating the toolkit field by calling
    the get_text/set_text methods, as opposed to having the toolkit
    object rely on change notification from the shell. The reason this
    is done this way is so that the majority of the conversion and
    validation logic can be managed by the shell, instead of repeating
    the same logic in every backend. 

    In order for the toolkit object to communicate successfully with
    the shell object, it must call three methods at various points
    in time:
        
        _field_text_edited
            This should be called when the user has edited the text in
            the UI, and that text has validated as either ACCEPTABLE
            or INTERMEDIATE using the shell's validator object. Text
            which validates as INVALID should be rejected. i.e. the
            text should not be allowed to be entered into the field.
            This means that validation must be performed before the
            new text is actually drawn on the screen.
        
        _field_return_pressed
            This should be called when the user presses the return key
            while the field has focus, but only if the text in the
            field validates as ACCEPTABLE. If return is pressed while
            the text is INTERMEDIATE, the 'normalize' method of the 
            shell's validator should be invoked and if the returned
            text validates as ACCEPTABLE, the new text should be used
            and this method called. If the 'normalize' method returns
            INVALID or INTERMEDIATE text, that text should be ignored
            and this method should not be called.
        
        _field_lost_focus
            This method should be called when the field loses focus.
            If the text in the field does not validate as ACCEPTABLE,
            then the 'normalize' method on the shell's validator should
            be invoked. If the returned text validates as ACCEPTABLE,
            it should be substituted. Otherwise, it should be ignored.
            In any cases, this method should always be called when the
            field loses focus.
    
    The shell object provides a 'validator' object with two methods which
    should be used by the toolkit implementation:
        
        validate
            This method accepts the text in the field and returns one
            of three results:
                
                validator.ACCEPTABLE
                    The text is acceptable input for the field.
                
                validator.INTERMEDIATE
                    The text can be made acceptable by the addition of
                    more characters.
                
                validator.INVALID
                    The text is clearly not valid and adding more 
                    characters cannot make it valid.
            
            When the user attempts to input a string which validates as
            INVALID, this input should be silently rejected before the 
            new characters are even drawn to the screen.
        
        normalize
            This method accepts the text in the field and returns a
            new string, which may or may not be different. This method
            allows the programmer a chance to fixup the user's input
            into a potentially acceptable value. The rules for when
            to call this method are described above.

    """
    @abstractmethod
    def get_text(self):
        """ Returns the current unicode text in the field.

        """
        raise NotImplementedError
    
    @abstractmethod
    def set_text(self, text):
        """ Sets the current unicode text in the field.

        """
        raise NotImplementedError
        
    @abstractmethod
    def shell_validator_changed(self, validator):
        """ The change handler for the 'validator' attribute on the 
        shell object.

        """
        raise NotImplementedError
    
    @abstractmethod
    def shell_max_length_changed(self, max_length):
        """ The change handler for the 'max_length' attribute on the 
        shell object.

        """ 
        raise NotImplementedError

    @abstractmethod
    def shell_read_only_changed(self, read_only):
        """ The change handler for the 'read_only' attribute on the 
        shell object.

        """
        raise NotImplementedError

    @abstractmethod
    def shell_cursor_position_changed(self, cursor_position):
        """ The change handler for the 'cursor_position' attribute on 
        the shell object.

        """
        raise NotImplementedError

    @abstractmethod
    def shell_placeholder_text_changed(self, placeholder_text):
        """ The change handler for the 'placeholder_text' attribute on 
        the shell object.

        """
        raise NotImplementedError
    
    @abstractmethod
    def shell_password_mode_changed(self, mode):
        """ The change handler for the 'password_mode' attribute on the
        shell object.

        """
        raise NotImplementedError

    @abstractmethod
    def set_selection(self, start, end):
        """ Sets the selection to the bounds of start and end.

        If the indices are invalid, no selection will be made, and any
        current selection will be cleared.

        Arguments
        ---------
        start : Int
            The start selection index, zero based.

        end : Int
            The end selection index, zero based.

        """
        raise NotImplementedError

    @abstractmethod
    def select_all(self):
        """ Select all the text in the field.

        If there is no text in the field, the selection will be empty.

        """
        raise NotImplementedError

    @abstractmethod
    def deselect(self):
        """ Deselect any selected text.

        """
        raise NotImplementedError

    @abstractmethod
    def clear(self):
        """ Clear the field of all text.

        """
        raise NotImplementedError

    @abstractmethod
    def backspace(self):
        """ Simple backspace functionality.

        If no text is selected, deletes the character to the left of 
        the cursor. Otherwise, it deletes the selected text.

        """
        raise NotImplementedError

    @abstractmethod
    def delete(self):
        """ Simple delete functionality.

        If no text is selected, deletes the character to the right of 
        the cursor. Otherwise, it deletes the selected text.

        """
        raise NotImplementedError

    @abstractmethod
    def end(self, mark=False):
        """ Moves the cursor to the end of the line.

        Arguments
        ---------
        mark : bool, optional
            If True, select the text from the current position to the 
            end of the field. Defaults to False.

        """
        raise NotImplementedError

    @abstractmethod
    def home(self, mark=False):
        """ Moves the cursor to the beginning of the line.

        Arguments
        ---------
        mark : bool, optional
            If True, select the text from the current position to the 
            beginning of the field. Defaults to False.

        """
        raise NotImplementedError

    @abstractmethod
    def cut(self):
        """ Cuts the selected text from the field.

        Copies the selected text to the clipboard then deletes the 
        selected text from the field.

        """
        raise NotImplementedError

    @abstractmethod
    def copy(self):
        """ Copies the selected text to the clipboard.

        """
        raise NotImplementedError

    @abstractmethod
    def paste(self):
        """ Paste the contents of the clipboard into the field.

        Inserts the contents of the clipboard into the field at the 
        current cursor position, replacing any selected text.

        """
        raise NotImplementedError

    @abstractmethod
    def insert(self, text):
        """ Insert the text into the field.

        Inserts the given text into the field at the current cursor 
        position, replacing any selected text.

        Arguments
        ---------
        text : str
            The text to insert into the field.

        """
        raise NotImplementedError

    @abstractmethod
    def undo(self):
        """ Undoes the last operation.

        """
        raise NotImplementedError

    @abstractmethod
    def redo(self):
        """ Redoes the last operation.

        """
        raise NotImplementedError


class Field(Control):
    """ A single-line editable text widget.

    Among many other attributes, a Field accepts a validator object
    which allows any arbitrary python object to be displayed and edited
    by the Field.

    """
    #: The Python value to display in the field. The default value
    #: is an empty string.
    value = Any(u'')

    #: A validator which manages validation/conversion to and from the
    #: widget's unicode text and the value attribute.
    validator = Instance(AbstractValidator, factory=CoercingValidator)

    #: The maximum length of the field in characters. The default value
    #: is Zero and indicates there is no maximum length.
    max_length = Int

    #: Whether or not the field is read only. Defaults to False.
    read_only = Bool(False)

    #: The position of the cursor in the field. Defaults to Zero.
    cursor_position = Int

    #: The grayed-out text to display if 'value' is empty and the
    #: widget doesn't have focus. Defaults to the empty string.
    placeholder_text = Unicode
    
    #: How to obscure password text in the field.
    password_mode = Enum('normal', 'password', 'silent')

    #: A boolean which is set to True if the user has changed the text
    #: from the ui, False otherwise. This is reset to False if the
    #: value is updated programmatically. This should be considered
    #: read-only and any assignment by the user will be ignored.
    modified = Bool(False)

    #: A unicode attribute that is updated with the text selected in the
    #: field. This should be considered read-only and any assignment by 
    #: the user will be ignored. Use the various selection methods to
    #: programmatically change the selection.
    selected_text = Unicode

    #: A boolean indicating whether or not the text typed by the user 
    #: is acceptable for conversion as indicated by the validator. Note
    #: that this is distinctly different from the 'error' attribute which
    #: indicates an error occured while converting acceptable text to the
    #: Python value. This should be considered read-only and assignment 
    #: by the user will be ignored.
    acceptable = Bool
    def _acceptable_default(self):
        v = self.validator
        return v.validate(v.format(self.value)) == v.ACCEPTABLE

    #: A list of strings which indicates when the text the in the field
    #: should be converted to a Python object representation and stored 
    #: in the 'value' attribute. The default mode triggers submissions 
    #: on lost focus events and return pressed events. Allowable modes 
    #: are 'always', 'lost_focus', and 'return_pressed'. A submission 
    #: can also be manually triggered by calling the 'submit()' method.
    submit_mode = List(
        Enum('lost_focus', 'return_pressed', 'always'),
        value=['lost_focus', 'return_pressed'],
    )

    #: Fired when the text is changed by the user explicitly through
    #: the ui but not programmatically. The event object will contain
    #: the text.
    text_edited = EnamlEvent

    #: Fired when the return/enter key is pressed in the field, provided 
    #: that the text in the field validates as ACCEPTABLE.
    return_pressed = EnamlEvent
    
    #: Fired when the widget has lost input focus.
    lost_focus = EnamlEvent
        
    #: How strongly a component hugs it's contents' width. Fields ignore 
    #: the width hug by default, so they expand freely in width.
    hug_width = 'ignore'

    #: Overridden parent class trait
    abstract_obj = Instance(AbstractTkField)

    def get_text(self):
        """ Get the current text as a unicode string.

        """
        return self.abstract_obj.get_text()

    def set_text(self, text):
        """ Set the current text without an event notification.

        """
        return self.abstract_obj.set_text(text)

    def set_selection(self, start, end):
        """ Sets the selection to the bounds of start and end.

        If the indices are invalid, no selection will be made, and any
        current selection will be cleared.

        Arguments
        ---------
        start : Int
            The start selection index, zero based.

        end : Int
            The end selection index, zero based.

        """
        self.abstract_obj.set_selection(start, end)

    def select_all(self):
        """ Select all the text in the field.

        If there is no text in the field, the selection will be empty.

        """
        self.abstract_obj.select_all()

    def deselect(self):
        """ Deselect any selected text.

        """
        self.abstract_obj.deselect()

    def clear(self):
        """ Clear the field of all text.

        """
        self.abstract_obj.clear()

    def backspace(self):
        """ Simple backspace functionality.

        If no text is selected, deletes the character to the left of 
        the cursor. Otherwise, it deletes the selected text.

        """
        self.abstract_obj.backspace()

    def delete(self):
        """ Simple delete functionality.

        If no text is selected, deletes the character to the right of
        the cursor. Otherwise, it deletes the selected text.

        """
        self.abstract_obj.delete()

    def end(self, mark=False):
        """ Moves the cursor to the end of the line.

        Arguments
        ---------
        mark : bool, optional
            If True, select the text from the current position to the 
            end of the field. Defaults to False.

        """
        self.abstract_obj.end(mark=mark)

    def home(self, mark=False):
        """ Moves the cursor to the beginning of the line.

        Arguments
        ---------
        mark : bool, optional
            If True, select the text from the current position to the 
            beginning of the field. Defaults to False.

        """
        self.abstract_obj.home(mark=mark)

    def cut(self):
        """ Cuts the selected text from the field.

        Copies the selected text to the clipthen deletes the selected
        text from the field.

        """
        self.abstract_obj.cut()

    def copy(self):
        """ Copies the selected text to the clipboard.

        """
        self.abstract_obj.copy()

    def paste(self):
        """ Paste the contents of the clipboard into the field.

        Inserts the contents of the clipboard into the field at the 
        current cursor position, replacing any selected text.

        """
        self.abstract_obj.paste()

    def insert(self, text):
        """ Insert the text into the field.

        Inserts the given text into the field at the current cursor 
        position, replacing any selected text.

        Arguments
        ---------
        text : str
            The text to insert into the field.

        """
        self.abstract_obj.insert(text)

    def undo(self):
        """ Undoes the last operation.

        """
        self.abstract_obj.undo()

    def redo(self):
        """ Redoes the last operation.

        """
        self.abstract_obj.redo()
    
    #--------------------------------------------------------------------------
    # Submission Machinery
    #--------------------------------------------------------------------------
    def submit(self, format=True):
        """ A method which is called to perform the submission. 

        A submit involves converting the current text in the field into a 
        Python value using the current validator, and storing that value
        in the 'value' attribute. If an error occurs during conversion,
        then the 'invalid', 'error' and 'exception' attributes on the 
        field will be set appropriately.

        If the current text does not validate as ACCEPTABLE then the
        'value' attribute will not be changed. In such case if 'format'
        is True, then the existing value will be formatted and the 
        display text updated.
        
        Parameters
        ----------
        format : bool, optional
            Whether or not to re-format the text after conversion and
            submission. A False value is useful for submitting the value
            without causing the text in the field to be updated. The
            default is True.
        
        Returns
        -------
        result : bool
            True if the submission is successful, or False if the text
            is not valid or the conversion from text to value fails. 

        """
        text = self.get_text()
        v = self.validator
        self.acceptable = acceptable = (v.validate(text) == v.ACCEPTABLE)

        res = False
        if acceptable:
            try:
                value = v.convert(text)
            except ValueError as e:
                self.exception = e
                self.error = True
                return
            else:
                # Setting the value attribute may fire off a model 
                # subscription which has the potential to raise other
                # exceptions. XXX we may want to log this exception
                # at some point in the future.
                try:
                    with guard(self, 'submitting'):
                        self.value = value
                except Exception as e:
                    self.exception = e
                    self.error = True
                    return
                else:
                    res = True
                    self.exception = None
                    self.error = False

        if format:
            text = v.format(self.value)
            self.set_text(text)
            self.acceptable = (v.validate(text) == v.ACCEPTABLE)
        
        return res

    @on_trait_change('value, validator')
    def _update_text_from_value(self):
        """ A change handler which updates the displayed text whenever
        the 'value' or the 'validator' attributes change.

        """
        if not guard.guarded(self, 'submitting'):
            self.modified = False
            v = self.validator
            text = v.format(self.value)
            self.set_text(text)
            self.acceptable = (v.validate(text) == v.ACCEPTABLE)
    
    #--------------------------------------------------------------------------
    # Field Update Methods
    #--------------------------------------------------------------------------
    def _field_text_edited(self):
        """ A method which should be called by the toolkit implementation
        whenever the user edits the text in the ui, but not when the text
        has been updated by a call to set_text(). This method should not
        be called if the user input validates as INVALID as such text
        should be rejected entirely.

        """
        self.modified = True
        if 'always' in self.submit_mode:
            self.submit(format=False)
        else:
            v = self.validator
            self.acceptable = (v.validate(self.get_text()) == v.ACCEPTABLE)
        self.text_edited()
    
    def _field_return_pressed(self):
        """ A method which should be called by the toolkit implementation
        whenever the user presses the return key, but only if the text in 
        the field validates as ACCEPTABLE.

        """
        if 'return_pressed' in self.submit_mode:
            self.submit()
        self.return_pressed()
    
    def _field_lost_focus(self):
        """ A method which should be called by the toolkit implementation 
        whenever the field loses focus.

        """
        if 'lost_focus' in self.submit_mode:
            self.submit()
        self.lost_focus()




#------------------------------------------------------------------------------
# Copyright (c) 2011, Enthought, Inc.
# All rights reserved.
#------------------------------------------------------------------------------
from abc import abstractmethod

from traits.api import (
    List, Enum, Unicode, Bool, Instance, Property, cached_property,
)

from .container import Container
from .layout_task_handler import LayoutTaskHandler
from .widget_component import WidgetComponent, AbstractTkWidgetComponent

from ..core.trait_types import EnamlEvent


class AbstractTkDockPane(AbstractTkWidgetComponent):
    """ The abstract toolkit interface for a DockPane.

    """
    @abstractmethod
    def shell_title_changed(self, title):
        """ Updates the title bar text in the dock pane.

        """
        raise NotImplementedError
    
    @abstractmethod
    def shell_closable_changed(self, closable):
        """ Updates whether or not the dock pane should have a close
        button in the title bar.

        """
        raise NotImplementedError
    
    @abstractmethod
    def shell_movable_changed(self, movable):
        """ Updates whether or not the dock pane should be movable from
        its current position.

        """
        raise NotImplementedError
    
    @abstractmethod
    def shell_floatable_changed(self, floatable):
        """ Updates whether or not the dock pane is allowed to be 
        undocked an floated over the main window.

        """
        raise NotImplementedError
    
    @abstractmethod
    def shell_floating_changed(self, floating):
        """ Updates whether or no the dock pane is floating over
        the main window instead of docked in its area.

        """
        raise NotImplementedError

    @abstractmethod
    def shell_title_bar_orientation_changed(self, orientation):
        """ Updates the orientation of the title bar.

        """
        raise NotImplementedError

    @abstractmethod
    def shell_allowed_dock_areas_changed(self, allowed_areas):
        """ Updates the allowable docking areas for the dock pane.

        """
        raise NotImplementedError
    
    @abstractmethod
    def shell_dock_widget_changed(self, dock_widget):
        """ Updates the widget being managed by the dock pane.

        """
        raise NotImplementedError


class DockPane(LayoutTaskHandler, WidgetComponent):
    """ A DockPane is a widget which holds a single child component and 
    can be provided to a DockManager in a MainWindow so that it may
    be docked in the window's various docking areas.

    A dock pane contains a title bar with an optional close button.
    When a dock pane is closed, it is not destroyed. Rather it's 
    visibility is set to False and it consumes zero space in the 
    dock area. To fully remove a dock pane from a window, it must
    be removed from the dock manager.

    """
    #: The title of the dock area.
    title = Unicode

    #: Whether or not the dock area is closable via a close button.
    closable = Bool(True)

    #: Whether or not the dock area is movable from its current position.
    movable = Bool(True)

    #: Whether or not the dock are is floatable as a separate window.
    floatable = Bool(True)
    
    #: A boolean indicating whether or not the dock pane is floating.
    floating = Bool(False)

    #: The orientation of the title bar.
    title_bar_orientation = Enum('horizontal', 'vertical')

    #: The dock area in which the pane currently resides.
    dock_area = Enum('left', 'right', 'top', 'bottom')

    #: The areas in the main window in which this dock area is allowed
    #: to be docked via user interaction.
    allowed_dock_areas = List(
        Enum('left', 'right', 'top', 'bottom', 'all'), value=['all'],
    )

    #: The widget being managed by the dock area.
    dock_widget = Property(
        Instance(WidgetComponent), depends_on='children',
    )

    #: An event that is emitted whenever the dock pane is moved from
    #: its current dock location to another dock location. The event
    #: payload will be a tuple of (old_area, new_area). If the pane
    #: is moved to a new location in the same area, then the old
    #: area will be the same as the new area.
    moved = EnamlEvent

    #: An event that is emitted whenever the dock pane is undocked
    #: and becomes a floating pane.
    undocked = EnamlEvent

    #: An event that is emitted whenever the floating dock pane
    #: is docked in a dock area.
    docked = EnamlEvent 

    #: Overridden parent class trait
    abstract_obj = Instance(AbstractTkDockPane)
    
    #--------------------------------------------------------------------------
    # Property Getters
    #--------------------------------------------------------------------------
    @cached_property
    def _get_dock_widget(self):
        """ The property getter for the 'dock_widget' attribute.

        """
        flt = lambda child: isinstance(child, WidgetComponent)
        widgets = filter(flt, self.children)
        n = len(widgets)
        if n != 1:
            msg = ('A DockPane must have exactly 1 child widget. '
                   'Got %s instead.')
            raise ValueError(msg % n)
        else:
            res = widgets[0]
        return res

    #--------------------------------------------------------------------------
    # Setup Methods
    #--------------------------------------------------------------------------
    def _setup_finalize(self):
        """ Updates the minimum size of the underlying dock widget during
        the finalization pass of the setup.

        """
        super(DockPane, self)._setup_finalize()
        self._update_dock_widget_min_size()

    #--------------------------------------------------------------------------
    # Layout Handling
    #--------------------------------------------------------------------------
    def do_relayout(self):
        """ A reimplemented LayoutTaskHandler handler method which will
        perform necessary update activity when a relayout it requested.

        """
        self._update_dock_widget_min_size()

    #--------------------------------------------------------------------------
    # Auxiliary Methods
    #--------------------------------------------------------------------------
    def _update_dock_widget_min_size(self):
        """ Updates the minimum size of the underlying dock widget based
        on its computed value if it's a Container, or its size hint 
        otherwise.

        """
        widget = self.dock_widget
        if isinstance(widget, Container):
            min_size = widget.compute_min_size()
        else:
            min_size = widget.size_hint()
        widget.set_min_size(min_size)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from abc import abstractmethod

from traits.api import Instance

from .container import Container
from .menu_bar import MenuBar
from .window import Window, AbstractTkWindow

from ..core.trait_types import EnamlEvent, EnamlWidgetInstance
from ..layout.geometry import Size
from ..noncomponents.abstract_dock_manager import AbstractTkDockManager


class AbstractTkMainWindow(AbstractTkWindow):
    """ The abstract toolkit interface for a MainWindow.

    """
    @abstractmethod
    def shell_menu_bar_changed(self, menu_bar):
        """ Update the window's menu bar with the provide Enaml MenuBar
        instance.

        """
        raise NotImplementedError


class MainWindow(Window):
    """ A top-level main window widget.

    MainWindow widgets are top-level widgets which provide various frame 
    decorations and other window related functionality. A window may 
    optionally contain a menubar, any number of toolbars, a status bar,
    and dock panes. A window can have at most one central widget child
    which will be expanded to fit the available space of the window.
    
    Sizing information relates to the size of the central widget rather
    than the overall size of the window. That is, specifying a minimum
    size for a MainWindow is akin to specifying a minimum size for the
    central widget. The space consumed by dock panes and menus is in
    addition to this space.

    """
    #: The menu bar for the window. This widget is automatically setup
    #: and destroyed when assigned to this attribute. Instances should
    #: therefore not be reused, but created on-the-fly as needed.
    menu_bar = EnamlWidgetInstance(MenuBar)
    
    #: The dock manager used for this main window.
    dock_manager = Instance(AbstractTkDockManager)

    #: An event which is fired when the window is closed.
    closed = EnamlEvent

    #: Overridden parent class trait
    abstract_obj = Instance(AbstractTkMainWindow)

    #--------------------------------------------------------------------------
    # Setup Methods
    #--------------------------------------------------------------------------
    def _setup_finalize(self):
        """ A setup method which assigns the dock manager a reference 
        to this MainWindow instance.

        """
        super(MainWindow, self)._setup_finalize()
        self._set_dock_manager_window()

    #--------------------------------------------------------------------------
    # Change Handlers
    #--------------------------------------------------------------------------
    def _dock_manager_changed(self):
        """ The change handler for the 'dock_manager' attribute which 
        updates the dock manager with the current MainWindow reference.

        """
        self._set_dock_manager_window()
    
    #--------------------------------------------------------------------------
    # Overrides
    #--------------------------------------------------------------------------
    def resize_to_initial(self):
        """ Overridden parent class method which resizes the window to 
        the initial size according to the semantics required of a main
        window.

        If the value of the 'initial_size' attribute is (-1, -1), then
        the initiali size of the window is determined by using the size
        hint of the central widget. Otherwise, the given 'initial_size'
        is used. 

        Note: If any dock panels or toolbars are in use, then this 
           computed initial size will likely be smaller than the 
           allowable minimum size, and therefore the minimum size
           will end up being the initial size. This is usually 
           the desired behavior. If the off chance that it isn't
           then manually specifying an initial size *and* a minimum
           size is sufficient to override the default behavior.
        
        """
        if self.initialized:
            init_size = self.initial_size
            if init_size == (-1, -1):
                widget = self.central_widget
                if widget is not None:
                    init_size = widget.size_hint()
                    self.resize(Size(*init_size))
            else:
                self.resize(Size(*init_size))

    def update_minimum_size(self):
        """ Overridden parent class method which updates the minimum size
        according the semantics required for a main window.

        If the value of the 'minimum_size' attribute is (-1, -1), then
        the minimum size of the window is indirectly using the minimum
        size of the central widget. That is, the minimum size of the 
        main window will be large enough to accomodate the minimum size
        of the central widget plus whatever dock panels, menu bars, and
        tool bars are in use. If the 'minimum_size' attribute is set to
        something other than (-1, -1), then that value will be used as
        the minimum size for the *entire* window, without regard to any
        dock panels etc. that are in use. The default behavior is 
        usually what is desired in most applications.

        Note: The minimum size computation for a MainWindow does not
            make use of the 'minimum_size_default' attribute.

        """
        if self.initialized:
            min_size = self.minimum_size
            if min_size == (-1, -1):
                self._update_central_widget_min_size()
            else:
                self.set_min_size(Size(*min_size))

    def resize_to_minimum(self):
        """ Overridden parent class method which resizes the window to
        the minimum size, respecting the semantics of a main window.

        """
        if self.initialized:
            min_size = self.minimum_size
            if min_size == (-1, -1):
                min_size = self._update_central_widget_min_size()
                if min_size != (-1, -1):
                    self.resize(min_size)
            else:
                min_size = Size(*min_size)
                self.set_min_size(min_size)
                self.resize(min_size)

    def update_maximum_size(self):
        """ Overridden parent class method which updates the maximum size
        according the semantics required for a main window.

        If the value of the 'maximum_size' attribute is (-1, -1), then
        the maximum size of the window is limits allowed by most GUI 
        toolkits. That is, the maximum size of the central widget is 
        ignored, and it will be expanded to fit the available space in
        the window. This is distinctly different from the maximum size
        computation in a Window, where the maximum size of the central
        widget is respected. Respecting this max size in a MainWindow 
        is not supported since it would lead to unituitive interactions
        between the sizing of the central widget and any dock panels 
        that are in use.

        Note: The maximum size computation for a MainWindow does not
            make use of the 'minimum_size_default' attribute.

        """
        if self.initialized:
            max_size = self.maximum_size
            if max_size == (-1, -1):
                v = 2**24 - 1
                max_size = (v, v)
            self.set_max_size(Size(*max_size))

    def resize_to_maximum(self):
        """ Overridden parent class method which resizes the window to
        the maximum size, respecting the semantics of a main window.

        """
        if self.initialized:
            max_size = self.maximum_size
            if max_size == (-1, -1):
                v = 2**24 - 1
                max_size = (v, v)
            max_size = Size(*max_size)
            self.set_max_size(max_size)
            self.resize(max_size)
    
    #--------------------------------------------------------------------------
    # Helper Methods
    #--------------------------------------------------------------------------
    def _set_dock_manager_window(self):
        """ Supplies the current dock manager with a reference to this
        MainWindow instance.

        """
        mgr = self.dock_manager
        if mgr is not None:
            mgr.main_window = self

    def _update_central_widget_min_size(self):
        """ Computes, applies, and returns the minimum size for the 
        central widget.

        Returns
        -------
        result : Size
            The computed minimum Size instance for the central widget.
            If the central widget is None, the return value will be 
            Size(-1, -1).

        """
        widget = self.central_widget
        if widget is not None:
            if isinstance(widget, Container):
                min_size = Size(*widget.compute_min_size())
            else:
                min_size = Size(*widget.size_hint())
            widget.set_min_size(min_size)
        else:
            min_size = Size(-1, -1)
        return min_size




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .qt import QtGui
from .qt_constraints_widget import QtConstraintsWidget

from ...components.tab_group import AbstractTkTabGroup


#: A mapping from TabPosition enum values to qt tab positions.
_TAB_POSITION_MAP = {
    'top': QtGui.QTabWidget.North,
    'bottom': QtGui.QTabWidget.South,
    'left': QtGui.QTabWidget.West,
    'right': QtGui.QTabWidget.East,
}


class QtTabGroup(QtConstraintsWidget, AbstractTkTabGroup):
    """ A Qt implementation of TabGroup.

    """
    # Simpilar to QtGroupBox, don't use a widget item to compute layout
    # geometry, or layouts dont get enough space between other controls
    # and the tab widget.
    use_widget_item_for_layout = False

    #--------------------------------------------------------------------------
    # Setup Methods 
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Create the underlying QTabWidget control.

        """
        self.widget = QtGui.QTabWidget(parent)

    def initialize(self):
        """ Initialize the attributes of the QTabWidget.

        """
        super(QtTabGroup, self).initialize()
        self.update_tabs()
        shell = self.shell_obj
        self.set_selected_index(shell.selected_index)
        self.set_tab_position(shell.tab_position)

    def bind(self):
        """ Bind to the events emitted by the underlying control.

        """
        super(QtTabGroup, self).bind()
        self.widget.currentChanged.connect(self._on_current_changed)

    #--------------------------------------------------------------------------
    # Abstract Implementation 
    #--------------------------------------------------------------------------
    def shell_tabs_changed(self, tabs):
        """ The change handler for the 'tabs' attribute of the shell 
        object.

        """
        self.update_tabs()

    def shell_tab_position_changed(self, tab_position):
        """ The change handler for the 'tab_position' attribute of the
        shell object.

        """
        self.set_tab_position(tab_position)

    def shell_selected_index_changed(self, index):
        """ Update the widget index with the new value from the shell 
        object.

        """
        self.set_selected_index(index)

    #--------------------------------------------------------------------------
    # Event Handlers 
    #--------------------------------------------------------------------------
    def _on_current_changed(self):
        """ The event handler for the 'currentChanged' signal of the 
        underlying control. Synchronizes the index of the shell object.

        """
        self.shell_obj._selected_index = self.widget.currentIndex()

    #--------------------------------------------------------------------------
    # Widget Update Methods 
    #--------------------------------------------------------------------------
    def set_tab_position(self, tab_position):
        """ Sets the position of the tabs on the underlying tab widget.

        """
        q_tab_position = _TAB_POSITION_MAP[tab_position]
        self.widget.setTabPosition(q_tab_position)

    def set_selected_index(self, index):
        """ Sets the current index of the tab widget. This is overridden
        from the parent class.

        """
        self.widget.setCurrentIndex(index)

    def update_tabs(self):
        """ Update the QTabWidget's children with the current children.
        This is an overridden parent class method.

        """
        widget = self.widget
        shell = self.shell_obj
        widget.clear()

        selected = shell.selected_tab
        for idx, tab in enumerate(shell.tabs):
            widget.addTab(tab.toolkit_widget, tab.title)
            if tab is selected:
                widget.setCurrentIndex(idx)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from abc import abstractmethod

from traits.api import (
    Str, Instance, Property, cached_property, Tuple, Range,
)

from .container import Container
from .layout_task_handler import LayoutTaskHandler
from .widget_component import WidgetComponent, AbstractTkWidgetComponent

from ..guard import guard
from ..layout.geometry import Size
from ..noncomponents.abstract_icon import AbstractTkIcon


# The set of strengths that are equal to, or stronger than, the resize
# strength of the window. This affects how the minimum and maximum 
# sizes of the window are computed.
STRONGER_THAN_RESIZE = set(('medium', 'strong', 'required'))


class AbstractTkWindow(AbstractTkWidgetComponent):
    """ The abstract ToplevelWindow interface.

    """
    @abstractmethod
    def maximize(self):
        """ Maximizes the window to fill the screen.

        """
        raise NotImplementedError
    
    @abstractmethod
    def minimize(self):
        """ Minimizes the window to the task bar.

        """
        raise NotImplementedError
    
    @abstractmethod
    def restore(self):
        """ Restores the window after it has been minimized or maximized.

        """
        raise NotImplementedError

    @abstractmethod
    def shell_title_changed(self, title):
        """ Update the title of the window with the new value from the
        shell object.

        """
        raise NotImplementedError

    @abstractmethod
    def shell_icon_changed(self, icon):
        """ Update the icon of the window with the new value from the
        shell object.

        """
        raise NotImplementedError

    @abstractmethod
    def shell_central_widget_changed(self, central_widget):
        """ Update the central widget in the window with the new value
        from the shell object.

        """
        raise NotImplementedError


class Window(LayoutTaskHandler, WidgetComponent):
    """ A top-level Window component.

    A Window component is represents of a top-level visible component
    with a frame decoration. It may have at most one child widget which
    is expanded to fit the size of the window. It does not support
    features like MenuBars or DockPanes, for that, use a MainWindow.

    """
    #: The title displayed on the window frame.
    title = Str

    #: The icon displayed on the frame next to the title.
    icon = Instance(AbstractTkIcon)

    #: A read-only property which holds the central widget. Declaring
    #: more than one central widget is an error.
    central_widget = Property(
        Instance(WidgetComponent), depends_on='children',
    )
    
    #: The initial size to set the window on the first call show()
    #: If the initial size is set to (-1, -1), then a default value
    #: will be computed more-or-less intelligently. This attribute
    #: is only used the first time the window is shown to the screen.
    #: Further changes to the attribute will have no effect. The
    #: constituent values must be >= -1. The default is (-1, -1).
    initial_size = Tuple(Range(-1), Range(-1))
    
    #: The initial size of the window to use if the attempt to compute
    #: one intelligently fails. This is the "last resort" of determining
    #: the initial size. The constituent values must be >= 0. The 
    #: default is (200, 100).
    initial_size_default = Tuple(Range(0, value=200), Range(0, value=100))

    #: The minimum size allowable for the window. If the min size is 
    #: set to (-1, -1), then the minimum will be computed based on
    #: the children of the window, and in such cases the minimum size
    #: that is in use can be retrieved with the min_size() method.
    #: The constituent values must be >= -1. The default is (-1, -1).
    minimum_size = Tuple(Range(-1), Range(-1))

    #: The minimum size of the window to use if the attempt to compute
    #: one intelligently fails. This is the "last resort" of determining
    #: the minimum size. The constituent values must be >= 0. The 
    #: default is (0, 0).
    minimum_size_default = Tuple(Range(0), Range(0))

    #: The maximum size allowable for the window. If the max size is 
    #: set to (-1, -1), then the maximum will be computed based on
    #: the children of the window, and in such cases the maximum size
    #: that is in use can be retrieved with the max_size() method.
    #: The consituent values must be >= -1. The default is (-1, -1).
    maximum_size = Tuple(Range(-1), Range(-1))

    #: The maximum size of the window to use if the attempt to compute
    #: one intelligently fails. This is the "last resort" of determining
    #: the minimum size. The constituent values must be >= 0. The 
    #: default is (2**24 - 1, 2**24 - 1)
    maximum_size_default = Tuple(Range(0, value=(2**24 - 1)), 
                                 Range(0, value=(2**24 - 1)))

    #: Overridden parent class trait
    abstract_obj = Instance(AbstractTkWindow)

    #--------------------------------------------------------------------------
    # Property Getters
    #--------------------------------------------------------------------------
    @cached_property
    def _get_central_widget(self):
        """ The property getter for the 'central_widget' attribute.

        """
        flt = lambda child: isinstance(child, WidgetComponent)
        widgets = filter(flt, self.children)
        n = len(widgets)
        if n > 1:
            msg = ('A Window can have at most 1 central widget. '
                   'Got %s instead.')
            raise ValueError(msg % n)
        elif n == 0:
            res = None
        else:
            res = widgets[0]
        return res
    
    #--------------------------------------------------------------------------
    # Overrides
    #--------------------------------------------------------------------------
    def set_visible(self, visible):
        """ Set the visibility of the component according to the given
        boolean. This is overridden from the parent class to properly
        handle visibility for toplevel components.

        """
        # Make sure the 'visible' attribute is synced up as a result
        # of the method call. This may fire a notification, in which
        # case the change handler will call this method again. This
        # guard prevents that unneeded recursion.
        if guard.guarded(self, 'set_visible'):
            return
        else:
            with guard(self, 'set_visible'):
                self.visible = visible
        
        # Only set the visibility to True (which will show the window) 
        # if the component is fully initialized.
        if not visible or self.initialized:
            self.abstract_obj.set_visible(visible)

    #--------------------------------------------------------------------------
    # Change Handlers
    #--------------------------------------------------------------------------
    def _minimum_size_changed(self):
        """ Resets the minimum size of the window if the user provides
        a new value after initialization.

        """
        self.update_minimum_size()
    
    def _maximum_size_changed(self):
        """ Resets the maximum size of the window if the user provides
        a new value after initialization.

        """
        self.update_maximum_size()

    def _central_widget_changed(self):
        """ A change handler for triggering a relayout when the central
        widget changes, provided that the component is initialized.

        """
        if self.initialized:
            self.request_relayout()

    #--------------------------------------------------------------------------
    # Layout Handling
    #--------------------------------------------------------------------------
    def do_relayout(self):
        """ A reimplemented LayoutTaskHandler handler method which will
        perform necessary update activity when a relayout it requested.

        """
        # This method is called whenever a relayout is requested. By
        # default, this is when the layout children change. In that case
        # we just need to update the min and max sizes. We are a top
        # level window, so no one really cares about our size hint. 
        self.update_minimum_size()
        self.update_maximum_size()

    #--------------------------------------------------------------------------
    # Auxiliary Methods
    #--------------------------------------------------------------------------
    def show(self, parent=None):
        """ Make the window visible on the screen.

        If the window is not already fully initialized, then the 'setup'
        method will be called prior to making the window visible.

        Parameters
        ----------
        parent : native toolkit widget, optional
            Provide this argument if the window should have another
            widget as its logical parent. This may help with stacking
            order and/or visibility hierarchy depending on the toolkit
            backend.

        """
        # Some Gui's don't like to process all events from a single 
        # call to process events (Qt), and pumping the loop is not
        # reliable. Instead, we just schedule the call to set_visible 
        # to occur after we start the event loop and with a priority 
        # that is less than any relayouts the may be triggered by 
        # pending events. This means that the layout queue should 
        # finish processing, and then the window will be shown.
        self._prep_window()
        app = self.toolkit.app
        app.schedule(self.set_visible, (True,), priority=75)
        app.start_event_loop()
        
    def hide(self):
        """ Hide the window, but do not destroy the underlying widgets.

        """
        self.set_visible(False)

    def maximize(self):
        """ Maximizes the window to fill the screen.

        """
        self.abstract_obj.maximize()
            
    def minimize(self):
        """ Minimizes the window to the task bar.

        """
        self.abstract_obj.minimize()
            
    def restore(self):
        """ Restores the window after it has been minimized or maximized.

        """
        self.abstract_obj.restore()

    def resize_to_initial(self):
        """ Resizes the window to the computed initial size, provided 
        that the window has been initialized.

        """
        if self.initialized:
            init_size = self._compute_initial_size()
            self.resize(init_size)

    def update_minimum_size(self):
        """ Updates the minimum size of the window based on its computed
        value, provided that the window has been initialized. This may 
        result in a resize if the computed value is more than the current
        size.

        """
        if self.initialized:
            min_size = self._compute_minimum_size()
            self.set_min_size(min_size)

    def resize_to_minimum(self):
        """ Resizes the window to the computed minimum size, provided 
        that the window has been initialized.

        Note: This is not equivalent to making the window minimize
            into the task bar. For that, use the 'minimize()' method.

        """
        if self.initialized:
            min_size = self._compute_minimum_size()
            self.set_min_size(min_size)
            self.resize(min_size)
    
    def update_maximum_size(self):
        """ Updates the maximum size of the window based on its computed
        value, provided that the window has been initialized. This may 
        result in a resize if the computed value is less than the current
        size.

        """
        if self.initialized:
            max_size = self._compute_maximum_size()
            self.set_max_size(max_size)

    def resize_to_maximum(self):
        """ Resizes the window to the computed maximum size, provided 
        that the window has been initialized. 

        Note: This is a potentially very expensive operation if the 
            maximum size is very large. This is not equivalent to 
            resizin a window to fit the screen. For that, use the 
            'maximize()' method.

        """
        if self.initialized:
            max_size = self._compute_maximum_size()
            self.set_max_size(max_size)
            self.resize(max_size)
    
    #--------------------------------------------------------------------------
    # Helper Methods
    #--------------------------------------------------------------------------
    def _prep_window(self, parent=None):
        """ A helper method which will prepare the Window for showing
        by intializing the tookit application and setting up the widget.
        The 'parent' argument is the same as would be passed to 'show'.

        """
        self.toolkit.app.initialize()
        if not self.initialized:
            self.setup(parent)
            self.resize_to_initial()
            self.update_minimum_size()
            self.update_maximum_size()
    
    def _compute_initial_size(self):
        """ Computes and returns the initial size of the window without
        regard for minimum or maximum sizes.

        """
        # If the user has supplied an explicit initial size, use that.
        computed_width, computed_height = self.initial_size
        if computed_width != -1 and computed_height != -1:
            return Size(computed_width, computed_height)
        
        # Otherwise, try to compute a default from the central widget.
        widget = self.central_widget
        if widget is not None:
            size_hint_width, size_hint_height = widget.size_hint()
            if computed_width == -1:
                computed_width = size_hint_width
            if computed_height == -1:
                computed_height = size_hint_height

        # We use the last resort values to replace any remaining 
        # -1 values. This ensures the return value will be >= 0 
        # in both width and height.
        if computed_width == -1 or computed_height == -1:
            default_width, default_height = self.initial_size_default
            if computed_width == -1:
                computed_width = default_width
            if computed_height == -1:
                computed_height = default_height

        return Size(computed_width, computed_height)

    def _compute_minimum_size(self):
        """ Computes and returns the minimum size of the window.

        """
        # If the user has supplied an explicit minimum size, use that.
        computed_width, computed_height = self.minimum_size
        if computed_width != -1 and computed_height != -1:
            return Size(computed_width, computed_height)
        
        # Otherwise, try to compute a default from the central widget.
        widget = self.central_widget
        if widget is not None:

            # If the central widget is a container, we have it compute
            # the minimum size for us, otherwise, we use the size hint
            # of the widget as the value.
            if isinstance(widget, Container):
                min_width, min_height = widget.compute_min_size()
            else:
                min_width, min_height = widget.size_hint()

            # If the hug and resist clip policies of the widget are
            # weaker than the resize strength of the window, then
            # we ignore its value in that direction.
            if ((widget.hug_width not in STRONGER_THAN_RESIZE) and
                (widget.resist_clip_width not in STRONGER_THAN_RESIZE)):
                min_width = -1
            
            if ((widget.hug_height not in STRONGER_THAN_RESIZE) and
                (widget.resist_clip_height not in STRONGER_THAN_RESIZE)):
                min_height = -1 

            if computed_width == -1:
                computed_width = min_width

            if computed_height == -1:
                computed_height = min_height
        
        # We use the last resort values to replace any remaining 
        # -1 values. This ensures the return value will be >= 0 
        # in both width and height
        if computed_width == -1 or computed_height == -1:
            default_width, default_height = self.minimum_size_default
            if computed_width == -1:
                computed_width = default_width
            if computed_height == -1:
                computed_height = default_height
        
        return Size(computed_width, computed_height)

    def _compute_maximum_size(self):
        """ Computes and returns the maximum size of the window.

        """
        # If the user has supplied an explicit maximum size, use that.
        computed_width, computed_height = self.maximum_size
        if computed_width != -1 and computed_height != -1:
            return (computed_width, computed_height)
        
        # Otherwise, try to compute a default from the central widget.
        widget = self.central_widget
        if widget is not None:

            # If the central widget is a container, we have it compute
            # the maximum size for us, otherwise, we use the size hint
            # of the widget as the value.
            if isinstance(widget, Container):
                max_width, max_height = widget.compute_max_size()
            else:
                max_width, max_height = widget.size_hint()

            # If the hug policy of the widget is weaker than the 
            # resize strength of the window, then we ignore its 
            # value in that direction.
            if widget.hug_width not in STRONGER_THAN_RESIZE:
                max_width = -1
            
            if widget.hug_height not in STRONGER_THAN_RESIZE:
                max_height = -1 

            if computed_width == -1:
                computed_width = max_width

            if computed_height == -1:
                computed_height = max_height
        
        # We use the last resort values to replace any remaining 
        # -1 values. This ensures the return value will be >= 0 
        # in both width and height.
        if computed_width == -1 or computed_height == -1:
            default_width, default_height = self.maximum_size_default
            if computed_width == -1:
                computed_width = default_width
            if computed_height == -1:
                computed_height = default_height
        
        return Size(computed_width, computed_height)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
class Toolkit(dict):
    """ The Enaml Toolkit class which facilitates toolkit independent
    development. 

    The Toolkit is a dict subclass which is injected between the global 
    and builtin scopes of an executing an Enaml function or expression.
    The contents of the dictionary may consist of any useful objects
    that should be automatically in scope for executing Enaml code. 
    Typical content of a toolkit will include component constructors, 
    layout helpers, operators, and useful toolkit abstractions.

    """
    __stack__ = []

    __default_toolkit__ = None

    __toolkit_app__ = None

    @classmethod
    def active_toolkit(cls):
        """ A classmethod that returns the currently active toolkit,
        or the default toolkit if there is not active toolkit context.

        """
        stack = cls.__stack__
        if not stack:
            tk = cls.default_toolkit()
        else:
            tk = stack[-1]
        return tk

    @classmethod
    def default_toolkit(cls):
        """ A classmethod that returns the default toolkit, creating one
        if necessary.

        """
        tk = cls.__default_toolkit__
        if tk is None:
            from enaml import default_toolkit
            tk = cls.__default_toolkit__ = default_toolkit()
        return tk

    def __enter__(self):
        """ A context manager method that pushes this toolkit onto
        the active toolkit stack.

        """
        self.__stack__.append(self)

    def __exit__(self, exc_type, exc_value, traceback):
        """ A context manager method that pops this toolkit from the
        active toolkit stack.

        """
        self.__stack__.pop()

    def _get_toolkit_app(self):
        """ Returns the abstract toolkit app for this toolkit or
        raises an ValueError if one is not defined.

        """
        return self.__toolkit_app__

    def _set_toolkit_app(self, val):
        """ Sets the abstract toolkit app for this toolkit and makes
        it available in the toolkit's namespace under the 'toolkit_app'
        name.

        """
        self.__toolkit_app__ = val

    app = property(_get_toolkit_app, _set_toolkit_app)




#------------------------------------------------------------------------------
# Copyright (c) 2011, Enthought, Inc.
# All rights reserved.
#------------------------------------------------------------------------------
from .qt import QtCore, QtGui
from .qt_constraints_widget import QtConstraintsWidget

from ...components.scroll_area import AbstractTkScrollArea


SCROLLBAR_POLICY_MAP = dict(
    as_needed=QtCore.Qt.ScrollBarAsNeeded,
    always_off=QtCore.Qt.ScrollBarAlwaysOff,
    always_on=QtCore.Qt.ScrollBarAlwaysOn,
)


class QtScrollArea(QtConstraintsWidget, AbstractTkScrollArea):
    """ Qt implementation of a ScrollArea.

    """
    #--------------------------------------------------------------------------
    # Setup methods
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Creates the underlying QScrollAreacontrol.

        """
        self.widget = QtGui.QScrollArea(parent)

    def initialize(self):
        """ Intializes the widget with the attributes of this instance.

        """
        super(QtScrollArea, self).initialize()
        self.widget.setWidgetResizable(True)
        shell = self.shell_obj
        self.set_horizontal_policy(shell.horizontal_scrollbar_policy)
        self.set_vertical_policy(shell.vertical_scrollbar_policy)
        self.update_scrolled_component()

    #--------------------------------------------------------------------------
    # Shell Object Change Handlers
    #--------------------------------------------------------------------------
    def shell_horizontal_scrollbar_policy_changed(self, policy):
        """ The change handler for the 'horizontal_scrollbar_policy'
        attribute of the shell object.

        """
        self.set_horizontal_policy(policy)

    def shell_vertical_scrollbar_policy_changed(self, policy):
        """ The change handler for the 'vertical_scrollbar_policy'
        attribute of the shell object.

        """
        self.set_vertical_policy(policy)

    def shell_scrolled_component_changed(self, component):
        """ The change handler for the 'layout_children' attribute of 
        the shell object.

        """
        self.update_scrolled_component()

    def vertical_scrollbar_thickness(self):
        """ Returns the pixel thickness of the scrollbar.

        """
        return self._scrollbar_thickness(QtCore.Qt.Vertical)
    
    def horizontal_scrollbar_thickness(self):
        """ Returns the pixel thickness of the scrollbar.

        """
        return self._scrollbar_thickness(QtCore.Qt.Vertical)

    def scroll_to_position(self, position, margin):
        """ Scrolls the area such that position is visible with a
        minimum of margin points surrounding position.

        """
        widget = self.widget

        pos_x, pos_y = position
        margin_x, margin_y = margin

        widget.ensureVisible(pos_x, pos_y, margin_x, margin_y)

    #--------------------------------------------------------------------------
    # Widget Update Methods
    #--------------------------------------------------------------------------
    def set_horizontal_policy(self, policy):
        """ Set the horizontal scrollbar policy of the widget.

        """
        self.widget.setHorizontalScrollBarPolicy(SCROLLBAR_POLICY_MAP[policy])

    def set_vertical_policy(self, policy):
        """ Set the vertical scrollbar policy of the widget.

        """
        self.widget.setVerticalScrollBarPolicy(SCROLLBAR_POLICY_MAP[policy])
    
    def update_scrolled_component(self):
        """ Update the QScrollArea's children with the current children.

        """
        component = self.shell_obj.scrolled_component
        if component is None:
            self.widget.setWidget(None)
        else:
            self.widget.setWidget(component.toolkit_widget)

    #--------------------------------------------------------------------------
    # Auxiliary Methods
    #--------------------------------------------------------------------------
    def _scrollbar_thickness(self, orientation):
        """ Returns the thickness of a scrollbar for the given orientation.

        """
        style = self.widget.style()
        options = QtGui.QStyleOptionSlider()
        options.orientation = orientation
        return style.pixelMetric(style.PM_ScrollBarExtent, options)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .abstract_validator import AbstractValidator
from .coercing_validator import CoercingValidator
from .datetime_validators import (
    DateValidator, DatetimeValidator, TimeValidator,
)
from .number_expression_validators import (
    NumberExpressionValidator, IntegralNumberExpressionValidator,
    RealNumberExpressionValidator, ComplexNumberExpressionValidator,
    IntExpressionValidator, LongExpressionValidator, FloatExpressionValidator,
    ComplexExpressionValidator, BinExpressionValidator, OctExpressionValidator,
    HexExpressionValidator, LongBinExpressionValidator,
    LongOctExpressionValidator, LongHexExpressionValidator,
)
from .number_validators import (
    NumberValidator, IntegralNumberValidator, RealNumberValidator,
    ComplexNumberValidator, IntValidator, LongValidator, FloatValidator,
    ComplexValidator, BinValidator, OctValidator, HexValidator,
    LongBinValidator, LongOctValidator, LongHexValidator,
)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from enable.api import Window as EnableWindow

from .qt_control import QtControl

from ...components.enable_canvas import AbstractTkEnableCanvas


class QtEnableCanvas(QtControl, AbstractTkEnableCanvas):
    """ A Qt implementation of EnableCanvas

    """
    #: The EnableWindow that is created to hold the component
    window = None

    #--------------------------------------------------------------------------
    # Setup methods
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Creates an EnableWindow instance to hold the component and
        uses that instance control as the toolkit widget. This assumes
        that enable picks the appropriate toolkit backend.

        """
        component = self.shell_obj.component
        self.window = EnableWindow(parent, component=component)
        self.widget = self.window.control
        # XXX Enable window doesn't parent on Qt (this is already fixed in Enable trunk)
        self.widget.setParent(parent)

    def shell_component_changed(self, component):
        """ The change handler for the 'component' attribute on the 
        shell object.

        """
        raise NotImplementedError('changing components not yet supported')

    def size_hint(self):
        """ A reimplemented parent class method to retrieve the 
        size hint as the preferred size from the enable component.

        """
        return self.shell_obj.component.get_preferred_size()




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import weakref

from .qt.QtCore import QSize
from .qt_constraints_widget import QtConstraintsWidget
from .qt_resizing_widgets import QResizingFrame, QResizingWidget

from ...components.container import AbstractTkContainer


class _QResizingFrame(QResizingFrame):
    """ A subclass of QResizingFrame which calls back onto the shell
    Container to get the size hint. If that size hint is invalid, it
    falls back onto the Qt default.

    """
    def __init__(self, qt_container, *args, **kwargs):
        super(QResizingFrame, self).__init__(*args, **kwargs)
        self.qt_container = weakref.ref(qt_container)
    
    def sizeHint(self):
        """ Computes the size hint from the given Container, falling
        back on the default size hint computation if the Container 
        returns one that is invalid.

        """
        res = None
        qt_container = self.qt_container()
        if qt_container is not None:
            shell = qt_container.shell_obj
            if shell is not None:
                sh = shell.size_hint()
                if sh != (-1, -1):
                    res = QSize(*sh)
        if res is None:
            res = super(_QResizingFrame, self).sizeHint()
        return res


class QtContainer(QtConstraintsWidget, AbstractTkContainer):
    """ A Qt4 implementation of Container.

    """
    def create(self, parent):
        """ Creates the underlying Qt widget.

        """
        self.widget = _QResizingFrame(self, parent)

    def bind(self):
        """ Binds the signal handlers for the widget.

        """
        super(QtContainer, self).bind()
        widget = self.widget
        if isinstance(widget, QResizingWidget):
            widget.resized.connect(self.on_resize)

    def on_resize(self):
        """ Triggers a relayout of the shell object since the widget
        has been resized.

        """
        # Notice that we are calling refresh() here instead of 
        # request_refresh() since we want the refresh to happen
        # immediately. Otherwise the resize layouts will appear 
        # to lag in the ui. This is a safe operation since by the
        # time we get this resize event, the widget has already 
        # changed size. Further, the only geometry that gets set
        # by the layout manager is that of our children. And should
        # it be required to resize this widget from within the layout
        # call, then the layout manager will do that asynchronously.
        self.shell_obj.refresh()




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import unittest
from . import test_wx_window
from .. import dialog


@unittest.skip('Wx Dialog must be run in an event loop to work')
class TestWXDialog(test_wx_window.TestWXWindow, dialog.TestDialog):
    """ WXDialog tests. """

    def get_title(self, widget):
        """ Get a window's title.

        """
        return widget.GetTitle()

    def disable_showing(self, widget):
        """ Disable the actual display of the dialog window.

        """
        widget.ShowModal = lambda:None





#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import wx.html

from .wx_control import WXControl

from ...components.html import AbstractTkHtml


class WXHtml(WXControl, AbstractTkHtml):
    """ A wxPython implementation of Html

    """
    #--------------------------------------------------------------------------
    # Setup methods
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Creates the underlying wx.html.HtmlWindow.

        """

        self.widget = wx.html.HtmlWindow(parent)

    def initialize(self):
        """ Initializes the attributes of the control.

        """
        shell = self.shell_obj
        self.set_page_source(shell.source)

    #---------------------------------------------------------------------------
    # Implementation
    #---------------------------------------------------------------------------
    def shell_source_changed(self, source):
        """ The change handler for the 'source' attribute.
        """
        self.set_page_source(source)

    def set_page_source(self, source):
        """ Sets the page source for the underlying control.

        """
        self.widget.SetPage(source)

    def size_hint(self):
        """ Overridden parent class method to return a sensible size
        hint.

        """ 
        # wx returns (0, 0) for the best size of an HtmlWindow so we 
        # just return the same size hint as a QAbstractScrollArea
        return (256, 192)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from collections import deque

from traits.api import HasStrictTraits, Instance

from .abstract_application import ScheduledTask

from ..guard import guard


class LayoutTaskHandler(HasStrictTraits):
    """ A mixin class that provides a basic implementation of layout
    task processing. Subclasses that use the mixin must at least be
    subclasses of BaseComponet and should, in addition, provide a
    freeze() method which returns a context manager, inside which the
    queues will be emptied and the layout handlers called.

    Only classes which actually implement some form of layout handling 
    should inherit this class. Otherwise, layout requests will not
    properly propagate up the widget hierarchy.

    This class should be inherited before the other superclasses of
    the parent class, or the layout handlers will not be properly
    overridden.

    """
    #: A private ScheduledTask that is created when handling requests
    #: for a deferred relayout.
    _relayout_task = Instance(ScheduledTask)
    
    #: A private ScheduledTask that is created when handling requests
    #: for a deferred refresh
    _refresh_task = Instance(ScheduledTask)

    #: A private queue of callables to run in a freeze context before
    #: performing any relayout.
    _relayout_queue = Instance(deque, ())

    #: A private queue of callables to run in a freeze context before
    #: performing any refresh.
    _refresh_queue = Instance(deque, ())

    #--------------------------------------------------------------------------
    # Layout Handling
    #--------------------------------------------------------------------------
    def relayout(self):
        """ A reimplemented parent class method which triggers an update
        of the constraints and a new layout computation. This is called 
        whenever the layout children of the component should have their 
        layout relationships recomputed. The constraints update and 
        relayout occur immediately and are completed before the method 
        returns. If the component is not fully initialized, this is a 
        no-op.

        """
        if not self.initialized:
            return
        # Protect against relayout recursion.
        if not guard.guarded(self, 'relayout'):
            with guard(self, 'relayout'):
                with self.freeze():
                    queue = self._relayout_queue
                    if queue:
                        while queue:
                            callback, args, kwargs = queue.popleft()
                            callback(*args, **kwargs)
                    self.do_relayout()
    
    def refresh(self):
        """ Reimplemented parent class method which triggers a refresh
        of the layout children. The refresh occurs immediately and is 
        completed before the method returns. If the component is not 
        fully initialized, this is a no-op.

        """
        if not self.initialized:
            return
        # Protect against refresh recursion.
        if not guard.guarded(self, 'refresh'):
            with guard(self, 'refresh'):
                with self.freeze():
                    queue = self._refresh_queue
                    if queue:
                        while queue:
                            callback, args, kwargs = queue.popleft()
                            callback(*args, **kwargs)
                    self.do_refresh()

    def request_relayout(self):
        """ Reimplemented parent class method which triggers an update
        of the constraints and a layout refresh at some point in the 
        future. Mutliple calls to this method will be collapsed into
        a single effective relayout.

        """
        task = self._relayout_task
        if task is None or not task.pending():
            self._relayout_task = self.toolkit.app.schedule(self.relayout)

    def request_refresh(self):
        """ Reimplemented parent class method which triggers a refresh
        of the children at some point in the future. Mutliple calls to 
        this method will be collapsed into a single effective refresh.

        """
        task = self._refresh_task
        if task is None or not task.pending():
            self._refresh_task = self.toolkit.app.schedule(self.refresh)

    def request_relayout_task(self, callback, *args, **kwargs):
        """ Reimplemented parent class method which requests a relayout
        after adding the callback to an internal queue. The queue will
        be emptied from within a freeze context the next time a relayout
        occurs.

        """
        self._relayout_queue.append((callback, args, kwargs))
        self.request_relayout()
            
    def request_refresh_task(self, callback, *args, **kwargs):
        """ Reimplemented parent class method which requests a relayout
        after adding the callback to an internal queue. The queue will
        be emptied from within a freeze context the next time a relayout
        occurs.

        """
        self._refresh_queue.append((callback, args, kwargs))
        self.request_refresh()
    
    #--------------------------------------------------------------------------
    # Layout Implementation Handlers
    #--------------------------------------------------------------------------
    def do_relayout(self):
        """ A method that should be implemented by subclasses to actually
        perform the relayout. By default, this method does nothing.

        """
        pass
    
    def do_refresh(self):
        """ A method that should be implemented by subclasses to actually
        perform the refresh. By default, this method does nothing.

        """
        pass




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from abc import abstractmethod

from traits.api import Str, Instance, Bool

from .control import Control, AbstractTkControl


class AbstractTkLabel(AbstractTkControl):
    
    @abstractmethod
    def shell_text_changed(self, text):
        raise NotImplementedError
        
    @abstractmethod
    def shell_word_wrap_changed(self, word_wrap):
        raise NotImplementedError


class Label(Control):
    """ A simple read-only text display.

    """
    #: The text in the label.
    text = Str
    
    #: Whether or not the label should wrap around on long lines.
    #: This may not be supported by all toolkit backends (like Wx)
    #: and it may be necessary with those toolkits to insert newline 
    #: characters where necessary.
    word_wrap = Bool(False)

    #: How strongly a component hugs it's content. Labels hug their
    #: contents' width weakly by default.
    hug_width = 'weak'

    #: Overridden parent class trait
    abstract_obj = Instance(AbstractTkLabel)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from abc import abstractmethod

from traits.api import Bool, Unicode, Instance

from .base_widget_component import (
    BaseWidgetComponent, AbstractTkBaseWidgetComponent,
)

from ..core.trait_types import EnamlEvent
from ..noncomponents.abstract_icon import AbstractTkIcon


class AbstractTkAction(AbstractTkBaseWidgetComponent):
    """ The abstract toolkit Action interface.

    """

    @abstractmethod
    def shell_enabled_changed(self, enabled):
        """ The change handler for the 'enabled' attribute of the shell
        object. Sets the widget enabled according to the given boolean.

        """
        raise NotImplementedError

    @abstractmethod
    def shell_text_changed(self, text):
        """ The change handler for the 'text' attribute on the shell
        object.

        """
        raise NotImplementedError
        
    @abstractmethod
    def shell_status_tip_changed(self, status_tip):
        """ The change handler for the 'status_tip' attribute on the 
        shell object.

        """
        raise NotImplementedError
    
    @abstractmethod
    def shell_tool_tip_changed(self, tool_tip):
        """ The change handler for the 'tool_tip' attribute on the 
        shell object.

        """
        raise NotImplementedError
    
    @abstractmethod
    def shell_icon_changed(self, icon):
        """ The change handler for the 'icon' attribute on the 
        shell object.

        """
        raise NotImplementedError
    
    @abstractmethod
    def shell_description_changed(self, description):
        """ The change handler for the 'description' attribute on the 
        shell object.

        """
        raise NotImplementedError

    @abstractmethod
    def shell_checkable_changed(self, checkable):
        """ The change handler for the 'checkable' attribute on the 
        shell object.

        """
        raise NotImplementedError
    
    @abstractmethod
    def shell_checked_changed(self, checked):
        """ The change handler for the 'checked' attribute on the
        shell object.

        """
        raise NotImplementedError

    @abstractmethod
    def shell_separator_changed(self, separator):
        """ The change handler for the 'separator' attribute on the
        shell object.

        """
        raise NotImplementedError


class Action(BaseWidgetComponent):
    """ A declarative Enaml component which represents an action 
    associated with a menu or toolbar.

    """
    #: Whether or not the widget is enabled.
    enabled = Bool(True)

    #: The text label associate with the action.
    text = Unicode

    #: The text that is displayed in the status bar when the user
    #: hovers over the action.
    status_tip = Unicode

    #: The tool tip text to use for this action. Typically displayed
    #: as small label when the user hovers over the action.
    tool_tip = Unicode

    #: The an icon to display with this action when appropriate, such as
    #: when the action is a member of a menu or a toolbar.
    icon = Instance(AbstractTkIcon)

    #: The description text to use for this action, such as context
    #: sensitive help. This is typically more descriptive that the
    #: tool tip text.
    description = Unicode

    #: An event fired when the action is triggered by user interaction.
    triggered = EnamlEvent

    #: An event fired when the user hovers over the action.
    hovered = EnamlEvent

    #: An event fired when a checkable action changes its checked state.
    toggled = EnamlEvent

    #: Whether or not the action can be checked.
    checkable = Bool(False)

    #: Whether or not the action is checked. This value only has meaning
    #: if 'checkable' is set to True.
    checked = Bool(False)

    #: Whether or not the action should be considered a separator.
    separator = Bool(False)

    #: Overridden parent class trait
    abstract_obj = Instance(AbstractTkAction)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from abc import abstractmethod

from traits.api import Bool, Instance, Property

from .window import Window, AbstractTkWindow

from ..core.trait_types import EnamlEvent
from ..enums import DialogResult, Modality


class AbstractTkDialog(AbstractTkWindow):
    """ The abstract toolkit interface for a Dialog.

    """
    @abstractmethod
    def accept(self):
        """ Close the dialog and set the result to `accepted`.

        """
        raise NotImplementedError

    @abstractmethod
    def reject(self):
        """ Close the dialog and set the result to `rejected`.

        """
        raise NotImplementedError


class Dialog(Window):
    """ A Window subclass which adds modal style behavior.

    The basic dialog has no buttons, but provides methods for the
    accept and reject behavior for the dialog.

    """
    #: A read only property which will be True when the dialog is open, 
    #: False otherwise.
    active = Property(Bool, depends_on='_active')

    #: Fired when the dialog is opened.
    opened = EnamlEvent

    #: Fired when the dialog is closed. The event payload will be the 
    #: dialog result.
    closed = EnamlEvent

    #: A read only property which is set to the result of the dialog; 
    #: 'rejected' if rejected() was called or the window was closed via 
    #: the 'x' window button, 'accepted' if accept() was called. The 
    #: result is set before the 'closed' event is fired.
    result = Property(DialogResult, depends_on='_result')

    #: An enum which indicates the modality of the dialog. One of 
    #: 'application_modal' or 'window_modal'. The default value is 
    #: 'application_modal'. Changes to this attribute *after* the 
    #: dialog is shown will have no effect.
    modality = Modality

    #: Overridden parent class trait.
    abstract_obj = Instance(AbstractTkDialog)
    
    #: An internal trait used to store the active state of the dialog.
    _active = Bool(False)

    #: An internal trait used to store the result of the dialog.
    _result = DialogResult('rejected')
        
    #--------------------------------------------------------------------------
    # Property Getters
    #--------------------------------------------------------------------------
    def _get_active(self):
        """ The property getter for the 'active' attribute.

        """
        return self._active

    def _get_result(self):
        """ The property getter for the 'result' attribute.

        """
        return self._result

    #--------------------------------------------------------------------------
    # Abstract Method Implementations
    #--------------------------------------------------------------------------
    def show(self, parent=None):
        """ Make the dialog visible on the screen. Overridden from the
        parent class to properly handle Dialog semantics.

        If the dialog is not already fully initialized, then the 'setup'
        method will be called prior to making the dialog visible.

        Parameters
        ----------
        parent : native toolkit widget, optional
            Provide this argument if the dialog should have another
            widget as its logical parent. This may help with stacking
            order and/or visibility hierarchy depending on the toolkit
            backend.

        """
        # Note that we don't start the event loop after making a Dialog
        # visible. Dialogs are shown modally and typically start their 
        # own event loop. Thus, set_visble(True) will block and starting
        # an event loop after it returns may cause a dead lock.
        #
        # XXX pumping the event loop here is a bit of hack. The trick
        # we used on Window doesn't work here because the set visible 
        # call actually starts the event loop.
        self._prep_window()
        app = self.toolkit.app
        for _ in range(10):
            app.process_events()
        self.set_visible(True)

    #--------------------------------------------------------------------------
    # Auxiliary Methods
    #--------------------------------------------------------------------------
    def accept(self):
        """ Close the dialog and set the result to `accepted`.

        Call this method to trigger the same behavior as clicking on an
        Ok button.

        """
        self.abstract_obj.accept()

    def reject(self):
        """ Close the dialog and set the result to `rejected`.

        Call this method to trigger the same behavior as clicking on a
        Cancel button.

        """
        self.abstract_obj.reject()

